# -*- coding: utf-8 -*-
"""
Created on Mon May  2 15:36:03 2022

@author: Julian Ceddia
"""
"""
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!WARNING: RUNNING run_test() WILL CHANGE SETTINGS IN NANONIS. RUN IT ON A SIM!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
"""

import time
from nanonisTCP import nanonisTCP
from nanonisTCP.ZController import ZController

"""
Set up the TCP connection and interface
"""
def run_test(TCP_IP='127.0.0.1', TCP_PORT=6502, make_plot=False):
    
    # Listening Port: see Nanonis File>Settings>TCP Programming Interface
    NTCP = nanonisTCP(TCP_IP, TCP_PORT)                                         # Nanonis TCP interface
    
    zctrl = ZController(NTCP)                                                   # Nanonis Z-Controller Module
    
    try:
        """
        ZPos Set/Get
        """
        zctrl.ZPosSet(140e-9)
        zpos = zctrl.ZPosGet()
        print("Z-Controller ZPos")
        print("zpos: " + str(zpos) + " m")
        print("----------------------------------------------------------------------")
        
        """
        OnOff Set/Get
        """
        zctrl.OnOffSet(on=False)
        time.sleep(1)                                                           # Need to sleep otherwise it doesn't update in time
        z_status = zctrl.OnOffGet()
        print("Z-Controller Status")
        print("Controller: " + ["Off","On"][z_status])
        print("----------------------------------------------------------------------")
        
        """
        Setpnt Set/Get
        """
        zctrl.SetpntSet(27e-12)
        setpoint = zctrl.SetpntGet()
        print("Z-Controller Setpoint")
        print("Setpoint: " + str(setpoint) + " A")
        print("----------------------------------------------------------------------")
        
        """
        Gain Set/Get
        """
        zctrl.GainSet(3e-12, 155e-6)
        p_gain,time_constant,i_gain = zctrl.GainGet()
        print("Z-Controller Gain")
        print("proportional gain: " + str(p_gain))
        print("integral gain:     " + str(i_gain))
        print("time constant:     " + str(time_constant))
        print("----------------------------------------------------------------------")
        
        """
        Switch off delay Set/Get
        """
        zctrl.SwitchOffDelaySet(0.07575)
        delay = zctrl.SwitchOffDelayGet()
        print("Z-Controller Switch Off Delay")
        print("Switch off delay: " + str(delay) + " s")
        print("----------------------------------------------------------------------")
        
        """
        Tip Lift Set/Get
        """
        zctrl.TipLiftSet(19e-9)
        tip_lift = zctrl.TipLiftGet()
        print("Z-Controller Tip Lift")
        print("Tip Lift: " + str(tip_lift) + " m")
        print("----------------------------------------------------------------------")
        
        """
        Home Properties Set/Get
        """
        zctrl.HomePropsSet(1, 160e-9)
        abs_rel,home_pos = zctrl.HomePropsGet()
        print("Z-Controller Home Properties")
        print("Mode:     " + ["Absolute","Relative"][abs_rel])
        print("Home Pos: " + str(home_pos) + " m")
        print("----------------------------------------------------------------------")
        
        """
        Home
        """
        zctrl.Home()
        
        """
        Control List Get
        """
        controllers,active_controller = zctrl.CtrlListGet()
        print("Z-Controller Controller")
        print("Available Controllers: " + str(controllers))
        print("Active Controller:     " + controllers[active_controller])
        print("----------------------------------------------------------------------")
        
        """
        Withdraw Rate Set/Get
        """
        zctrl.WithdrawRateSet(20e-9)
        slew_rate = zctrl.WithdrawRateGet()
        print("Z-Controller Slew Rate")
        print("Slew Rate: " + str(slew_rate) + " m/s")
        print("----------------------------------------------------------------------")
        
        """
        Withdraw
        """
        time.sleep(0.5)
        zctrl.Withdraw(wait_until_finished=True)
        
        """
        Limits Enabled Set
        """
        zctrl.LimitsEnabledSet(limit_z_status=True)
        limit_z_status = zctrl.LimitsEnabledGet()
        print("Z-Controller Z Limits Enabled/Disabled")
        print("Z limits " + ["disabled","enabled"][limit_z_status])
        print("----------------------------------------------------------------------")
        
        """
        Limits Set/Get
        """
        zctrl.LimitsSet(200e-9, -210e-9)
        z_high_limit, z_low_limit = zctrl.LimitsGet()
        print("Z-Controller Z Limits")
        print("Z high limit: " + str(z_high_limit) + " m")
        print("Z low limit:  " + str(z_low_limit) + " m")
        print("----------------------------------------------------------------------")
        
        """
        Z Controller Status
        """
        _,status_string = zctrl.StatusGet()
        print("Z-Controller Status")
        print("Controller status: " + status_string)
        print("----------------------------------------------------------------------")
        
    except Exception as e:
        print(e)
    
    NTCP.close_connection()


# -*- coding: utf-8 -*-
"""
Created on Fri Oct 14 20:29:16 2022

@author: jced0001
"""

class LockIn:
    """
    Nanonis LockIn Module
    """
    def __init__(self,NanonisTCP):
        self.NanonisTCP = NanonisTCP
    
    def ModOnOffSet(self,modulator_number,lockin_onoff):
        """
        Turns the specified Lock-In modulator on or off.

        Parameters
        ----------
        modulator_number : The number that specifies which modulator to use. 
                           It starts from number 1 (=Modulator 1)
        lockin_onoff     : Turns the specified modulator on or off, where 0=Off
                           and 1=On

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.ModOnOffSet', body_size=8)
        
        ## arguments
        hex_rep += self.NanonisTCP.to_hex(modulator_number,4)
        hex_rep += self.NanonisTCP.to_hex(lockin_onoff,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
    
    def ModOnOffGet(self,modulator_number):
        """
        Returns if the specified Lock-In modulator is turned on or off.

        Parameters
        ----------
        modulator_number : The number that specifies which modulator to use. 
                           It starts from number 1 (=Modulator 1)

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.ModOnOffGet', body_size=4)
        
        hex_rep += self.NanonisTCP.to_hex(modulator_number,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response()
        
        lockin_onoff = self.NanonisTCP.hex_to_uint32(response[0:4])
        
        return lockin_onoff
    
    def ModSignalSet(self,modulator_number,modulator_signal_index):
        """
        Selects the modulated signal of the specified Lock-In modulator.

        Parameters
        ----------
        modulator_number : the number that specifies which modulator to use.
                           It starts from number 1 (=Modulator 1)
        modulator_signal_index : the signal index out of the list of 128 signals
                           available in the software.
                           To get a list of the available signals, use the
                           Signals.NamesGet function.

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.ModSignalSet', body_size=8)
        
        ## arguments
        hex_rep += self.NanonisTCP.to_hex(modulator_number,4)
        hex_rep += self.NanonisTCP.to_hex(modulator_signal_index,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
        
    def ModSignalGet(self,modulator_number):
        """
        Returns the modulated signal of the specified Lock-In modulator.

        Returns
        -------
        modulator_signal_index : the signal index out of the list of 128 signals
                          available in the software.
                          To get a list of the available signals, use the 
                          Signals.NamesGet function.

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.ModSignalGet', body_size=4)
        
        hex_rep += self.NanonisTCP.to_hex(modulator_number,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response()
        
        modulator_signal_index = self.NanonisTCP.hex_to_uint32(response[0:4])
        
        return modulator_signal_index
    
    def ModPhasRegSet(self,modulator_number,phase_register_index):
        """
        Sets the phase register index of the specified Lock-In modulator.
        Each modulator can work on any phase register (frequency). Use this 
        function to assign the modulator to one of the 8 available phase
        registers (index 1-8). Use the LockIn.ModPhaFreqSet function to set the
        frequency of the phase registers.
        
        Parameters
        ----------
        modulator_number : the number that specifies which modulator to use.
                           It starts from number 1. (=Modulator 1)
        phase_register_index : the index of the phase register of the specified
                               Lock-In modulator. Valid values are index 1 to 8
                              
        """
        hex_rep = self.NanonisTCP.make_header('LockIn.ModPhasRegSet', body_size=8)
        
        ## arguments
        hex_rep += self.NanonisTCP.to_hex(modulator_number,4)
        hex_rep += self.NanonisTCP.to_hex(phase_register_index,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
    
    def ModPhasRegGet(self,modulator_number):
        """
        Returns the phase register index of the specified Lock-In modulator.
        Each modulator can work on any phase register (frequency generator).
        Use the LockIn.ModPhaseRegFreqGet function to get the frequency of the 
        phase registers.

        Parameters
        ----------
        modulator_number : the number that specifies which modulator to use.
                           It starts from number 1. (=Modulator 1)

        Returns
        -------
        phase_register_index : the index of the phase register of the specified
                               Lock-In modulator. Valid values are index 1 to 8
        

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.ModPhasRegGet', body_size=4)
        
        hex_rep += self.NanonisTCP.to_hex(modulator_number,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response()
        
        phase_register_index = self.NanonisTCP.hex_to_uint32(response[0:4])
        
        return phase_register_index
    
    def ModHarmonicSet(self,modulator_number,harmonic):
        """
        Sets the harmonic of the specified Lock-In modulator.
        The modulator is bound to a phase register (frequency generator), but 
        it can work on harmonics. Harmonic 1 is the base frequency (the 
        frequency of the frequency generator).

        Parameters
        ----------
        modulator_number : the number that specifies which modulator to use.
                           It starts from number 1. (=Modulator 1)
        harmonic         : the harmonic of the specified Lock-In modulator.
                           Valid values start from 1 (=base frequency)

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.ModHarmonicSet', body_size=8)
        
        ## arguments
        hex_rep += self.NanonisTCP.to_hex(modulator_number,4)
        hex_rep += self.NanonisTCP.to_hex(harmonic,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
    
    def ModHarmonicGet(self,modulator_number):
        """
        Returns the harmonic of the specified Lock-In modulator.
        The modulator is bound to a phase register (frequency generator), but 
        it can work on harmonics. Harmonic 1 is the base frequency (the 
        frequency of the frequency generator).

        Parameters
        ----------
        modulator_number : the number that specifies which modulator to use.
                           It starts from number 1. (=Modulator 1)

        Returns
        -------
        harmonic         : the harmonic of the specified Lock-In modulator.
                           Valid values start from 1 (=base frequency)

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.ModHarmonicGet', body_size=4)
        
        hex_rep += self.NanonisTCP.to_hex(modulator_number,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response()
        
        harmonic = self.NanonisTCP.hex_to_uint32(response[0:4])
        
        return harmonic
    
    def ModPhasSet(self,modulator_number,phase_deg):
        """
        Sets the modulation phase offset of the specified Lock-In modulator.

        Parameters
        ----------
        modulator_number : the number that specifies which modulator to use.
                           It starts from number 1. (=Modulator 1)
        phase_deg        : the modulation phase offset of the specified Lock-In
                           modulator

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.ModPhasSet', body_size=8)
        
        ## arguments
        hex_rep += self.NanonisTCP.to_hex(modulator_number,4)
        hex_rep += self.NanonisTCP.float32_to_hex(phase_deg)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
    
    def ModPhasGet(self,modulator_number):
        """
        Returns the modulation phase offset of the specified Lock-In modulator.

        Parameters
        ----------
        modulator_number : the number that specifies which modulator to use.
                           It starts from number 1. (=Modulator 1)

        Returns
        -------
        phase_deg        : the modulation phase offset of the specified Lock-In
                           modulator

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.ModPhasGet', body_size=4)
        
        hex_rep += self.NanonisTCP.to_hex(modulator_number,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response()
        
        phase_deg = self.NanonisTCP.hex_to_float32(response[0:4])
        
        return phase_deg
    
    def ModAmpSet(self,modulator_number,amplitude):
        """
        Sets the modulation amplitude of the specified Lock-In modulator.
        
        Parameters
        ----------
        modulator_number : the number that specifies which modulator to use.
                           It starts from number 1. (=Modulator 1)
        amplitude        : the modulation amplitude of the specified Lock-In 
                           modulator

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.ModAmpSet', body_size=8)
        
        ## arguments
        hex_rep += self.NanonisTCP.to_hex(modulator_number,4)
        hex_rep += self.NanonisTCP.float32_to_hex(amplitude)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
    
    def ModAmpGet(self,modulator_number):
        """
        Returns the modulation amplitude of the specified Lock-In modulator.

        Parameters
        ----------
        modulator_number : the number that specifies which modulator to use.
                           It starts from number 1. (=Modulator 1)

        Returns
        -------
        amplitude        : the modulation amplitude of the specified Lock-In 
                           modulator

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.ModAmpGet', body_size=4)
        
        hex_rep += self.NanonisTCP.to_hex(modulator_number,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response()
        
        amplitude = self.NanonisTCP.hex_to_float32(response[0:4])
        
        return amplitude
        
    def ModPhasFreqSet(self,modulator_number,frequency):
        """
        Sets the frequency of the specified Lock-In phase register/modulator.
        The Lock-in module has a total of 8 frequency generators / phase 
        registers. Each modulator and demodulator can be bound to one of the 
        phase registers.
        This function sets the frequency of one of the phase registers.

        Parameters
        ----------
        modulator_number : the number that specifies which modulator to use.
                           It starts from number 1. (=Modulator 1)
        frequency        : the frequency of the specified Lock-In phase 
                           register (Hz)

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.ModPhasFreqSet', body_size=12)
        
        ## arguments
        hex_rep += self.NanonisTCP.to_hex(modulator_number,4)
        hex_rep += self.NanonisTCP.float64_to_hex(frequency)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
    
    def ModPhasFreqGet(self,modulator_number):
        """
        Returns the frequency of the specified Lock-In phase register/modulator
        The Lock-in module has a total of 8 frequency generators / phase 
        registers. Each modulator and demodulator can be bound to one of the 
        phase registers.
        This function gets the frequency of one of the phase registers.

        Parameters
        ----------
        modulator_number : the number that specifies which modulator to use.
                           It starts from number 1. (=Modulator 1)

        Returns
        -------
        frequency        : the frequency of the specified Lock-In phase 
                           register (Hz)

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.ModPhasFreqGet', body_size=4)
        
        hex_rep += self.NanonisTCP.to_hex(modulator_number,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response()
        
        frequency = self.NanonisTCP.hex_to_float64(response[0:8])
        
        return frequency
    
    def DemodSignalSet(self,demod_number,demod_signal_index):
        """
        Selects the demodulated signal of the specified Lock-In demodulator.

        Parameters
        ----------
        demod_number : The number that specifies which demodulator to use. It 
                       starts from number 1 (=Demodulator 1)
        demod_signal_index : The signal index out of the list of 128 signals
                             available in the software.
                             To get a list of the available signals, use the 
                             Signals.NamesGet function.

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.DemodSignalSet', body_size=8)
        
        ## arguments
        hex_rep += self.NanonisTCP.to_hex(demod_number,4)
        hex_rep += self.NanonisTCP.to_hex(demod_signal_index,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
    
    def DemodSignalGet(self,demod_number):
        """
        Returns the demodulated signal of the specified Lock-In demodulator.

        Parameters
        ----------
        demod_number : The number that specifies which demodulator to use. It 
                       starts from number 1 (=Demodulator 1)

        Returns
        -------
        demod_signal_index : The signal index out of the list of 128 signals
                             available in the software.
                             To get a list of the available signals, use the 
                             Signals.NamesGet function.

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.DemodSignalGet', body_size=4)
        
        hex_rep += self.NanonisTCP.to_hex(demod_number,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response()
        
        demod_signal_index = self.NanonisTCP.hex_to_int32(response[0:4])
        
        return demod_signal_index
        
    def DemodHarmonicSet(self,demod_number,harmonic):
        """
        Sets the harmonic of the specified Lock-In demodulator.
        The demodulator demodulates the input signal at the specified harmonic
        overtone of the frequency generator. Harmonic 1 is the base frequency
        (the frequency of the frequency generator).

        Parameters
        ----------
        demod_number : The number that specifies which demodulator to use. It 
                       starts from number 1 (=Demodulator 1)
        harmonic : The harmonic of the specified Lock-In demodulator. Valid 
                   values start from 1 (=base frequency)

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.DemodHarmonicSet', body_size=8)
        
        ## arguments
        hex_rep += self.NanonisTCP.to_hex(demod_number,4)
        hex_rep += self.NanonisTCP.to_hex(harmonic,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
    
    def DemodHarmonicGet(self,demod_number):
        """
        Returns the harmonic of the specified Lock-In demodulator.
        The demodulator demodulates the input signal at the specified harmonic
        overtone of the frequency generator. Harmonic 1 is the base frequency 
        (the frequency of the frequency generator).

        Parameters
        ----------
        demod_number : The number that specifies which demodulator to use. It 
                       starts from number 1 (=Demodulator 1)

        Returns
        -------
        harmonic : The harmonic of the specified Lock-In demodulator. Valid 
                   values start from 1 (=base frequency)

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.DemodHarmonicGet', body_size=4)
        
        hex_rep += self.NanonisTCP.to_hex(demod_number,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response()
        
        harmonic = self.NanonisTCP.hex_to_int32(response[0:4])
        
        return harmonic
    
    def DemodHPFilterSet(self,demod_number,hp_filter_order,cutoff):
        """
        Sets the properties of the high-pass filter applied to the demodulated 
        signal of the specified demodulator.
        
        The high-pass filter is applied on the demodulated signal before the
        actual demodulation. It is used to get rid of DC or low-frequency 
        components which could result in undesired components close to the 
        modulation frequency on the demodulator output signals (X,Y).

        Parameters
        ----------
        demod_number : The number that specifies which demodulator to use. It 
                       starts from number 1 (=Demodulator 1)
        hp_filter_order : The high-pass filter order. Valid values are from 
                          -1 to 8.
                          -1 = no change
                           0 = filter off.
        cutoff :    high-pass filter cutoff frequency in Hz, where 0 = no change.

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.DemodHPFilterSet', body_size=12)
        
        ## arguments
        hex_rep += self.NanonisTCP.to_hex(demod_number,4)
        hex_rep += self.NanonisTCP.to_hex(hp_filter_order,4)
        hex_rep += self.NanonisTCP.float32_to_hex(cutoff)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
    
    def DemodHPFilterGet(self,demod_number):
        """
        Returns the properties of the high-pass filter applied to the 
        demodulated signal of the specified demodulator.
        
        The high-pass filter is applied on the demodulated signal before the
        actual demodulation. It is used to get rid of DC or low-frequency 
        components which could result in undesired components close to the 
        modulation frequency on the demodulator output signals (X,Y).

        Parameters
        ----------
        demod_number : The number that specifies which demodulator to use. It 
                       starts from number 1 (=Demodulator 1)

        Returns
        -------
        hp_filter_order : The high-pass filter order. Valid values are from 
                          0 to 8.
                          0 = filter off.
        cutoff :    high-pass filter cutoff frequency in Hz.

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.DemodHPFilterGet', body_size=4)
        
        hex_rep += self.NanonisTCP.to_hex(demod_number,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response()
        
        hp_filter_order = self.NanonisTCP.hex_to_int32(response[0:4])
        cutoff          = self.NanonisTCP.hex_to_float32(response[4:8])
        
        return {"hp_filter_order" : hp_filter_order,
                "cutoff": cutoff}
    
    def DemodLPFilterSet(self,demod_number,lp_filter_order,cutoff):
        """
        Sets the properties of the low-pass filter applied to the demodulated
        signal of the specified demodulator.
        
        The low-pass filter is applied on the demodulator output signals (X,Y) 
        to remove undesired components. Lower cut-off frequency means better
        suppression of undesired frequency components, but longer response time
        (time constant) of the filter.

        Parameters
        ----------
        demod_number : The number that specifies which demodulator to use. It 
                       starts from number 1 (=Demodulator 1)
        lp_filter_order : The low-pass filter order. Valid values are from 
                          -1 to 8.
                          -1 = no change
                           0 = filter off
        cutoff : low-pass filter cutoff frequency in Hz
                 0 = no change

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.DemodLPFilterSet', body_size=12)
        
        ## arguments
        hex_rep += self.NanonisTCP.to_hex(demod_number,4)
        hex_rep += self.NanonisTCP.to_hex(lp_filter_order,4)
        hex_rep += self.NanonisTCP.float32_to_hex(cutoff)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
    
    def DemodLPFilterGet(self,demod_number):
        """
        Returns the properties of the low-pass filter applied to the
        demodulated signal of the specified demodulator.
        
        The low-pass filter is applied on the demodulator output signals (X,Y) 
        to remove undesired components. Lower cut-off frequency means better 
        suppression of undesired frequency components, but longer response time 
        (time constant) of the filter.

        Parameters
        ----------
        demod_number : The number that specifies which demodulator to use. It 
                       starts from number 1 (=Demodulator 1)

        Returns
        -------
        lp_filter_order : The low-pass filter order. Valid values are from 
                          0 to 8.
                          0 = filter off
        cutoff : low-pass filter cutoff frequency in Hz

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.DemodLPFilterGet', body_size=4)
        
        hex_rep += self.NanonisTCP.to_hex(demod_number,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response()
        
        lp_filter_order = self.NanonisTCP.hex_to_int32(response[0:4])
        cutoff          = self.NanonisTCP.hex_to_float32(response[4:8])
        
        return {"lp_filter_order" : lp_filter_order,
                "cutoff": cutoff}
    
    def DemodPhasRegSet(self,demod_number,phase_register_index):
        """
        

        Parameters
        ----------
        demod_number : The number that specifies which demodulator to use. It 
                       starts from number 1 (=Demodulator 1)
        phase_register_index : The index of the phase register of the specified 
                               Lock-In demodulator. Valid values are index 
                               1 to 8.

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.DemodPhasRegSet', body_size=8)
        
        ## arguments
        hex_rep += self.NanonisTCP.to_hex(demod_number,4)
        hex_rep += self.NanonisTCP.to_hex(phase_register_index,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
        
    def DemodPhasRegGet(self,demod_number):
        """
        Returns the phase register index of the specified Lock-In demodulator.
        Each demodulator can work on any phase register (frequency). Use the 
        LockIn.ModPhaFreqSet function to set the frequency of the phase 
        registers.

        Parameters
        ----------
        demod_number : The number that specifies which demodulator to use. It 
                       starts from number 1 (=Demodulator 1)

        Returns
        -------
        phase_register_index : The index of the phase register of the specified 
                               Lock-In demodulator. Valid values are index 
                               1 to 8.

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.DemodPhasRegGet', body_size=4)
        
        hex_rep += self.NanonisTCP.to_hex(demod_number,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response()
        
        phase_register_index = self.NanonisTCP.hex_to_int32(response[0:4])
        
        return phase_register_index
    
    def DemodPhasSet(self,demod_number,phase_deg):
        """
        Sets the reference phase of the specified Lock-In demodulator.

        Parameters
        ----------
        demod_number : The number that specifies which demodulator to use. It 
                       starts from number 1 (=Demodulator 1)
        phase_deg : is the reference phase of the specified Lock-In demodulator

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.DemodPhasSet', body_size=8)
        
        ## arguments
        hex_rep += self.NanonisTCP.to_hex(demod_number,4)
        hex_rep += self.NanonisTCP.float32_to_hex(phase_deg)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
    
    def DemodPhasGet(self,demod_number):
        """
        Returns the reference phase of the specified Lock-In demodulator.

        Parameters
        ----------
        demod_number : The number that specifies which demodulator to use. It 
                       starts from number 1 (=Demodulator 1)

        Returns
        -------
        phase_deg : is the reference phase of the specified Lock-In demodulator

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.DemodPhasGet', body_size=4)
        
        hex_rep += self.NanonisTCP.to_hex(demod_number,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response()
        
        phase_deg = self.NanonisTCP.hex_to_float32(response[0:4])
        
        return phase_deg
    
    def DemodSyncFilterSet(self,demod_number,sync_filter):
        """
        Switches the synchronous (Sync) filter of the specified demodulator 
        On or Off.
        
        The synchronous filter is applied on the demodulator output signals 
        (X,Y) after the low-pass filter. It is very good in suppressing 
        harmonic components (harmonics of the demodulation frequency), but does 
        not suppress other frequencies.
        
        The sync filter does not output a continuous signal, it only updates 
        the value after each period of the demodulation frequency.

        Parameters
        ----------
        demod_number : The number that specifies which demodulator to use. It 
                       starts from number 1 (=Demodulator 1)
        sync_filter : The synchronous filter of the specified demodulator on 
                      or off.
                      0 = Off
                      1 = On

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.DemodSyncFilterSet', body_size=8)
        
        ## arguments
        hex_rep += self.NanonisTCP.to_hex(demod_number,4)
        hex_rep += self.NanonisTCP.to_hex(sync_filter,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
    
    def DemodSyncFilterGet(self,demod_number):
        """
        Returns the status (on/off) of the synchronous (Sync) filter of the
        specified demodulator.
        
        The synchronous filter is applied on the demodulator output signals 
        (X,Y) after the low-pass filter. It is very good in suppressing 
        harmonic components (harmonics of the demodulation frequency), but does
        not suppress other frequencies.
        
        The sync filter does not output a continuous signal, it only updates 
        the value after each period of the demodulation frequency.

        Parameters
        ----------
        demod_number : The number that specifies which demodulator to use. It 
                       starts from number 1 (=Demodulator 1)

        Returns
        -------
        sync_filter : The synchronous filter of the specified demodulator on 
                      or off.
                      0 = Off
                      1 = On

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.DemodSyncFilterGet', body_size=4)
        
        hex_rep += self.NanonisTCP.to_hex(demod_number,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response()
        
        sync_filter = self.NanonisTCP.hex_to_uint32(response[0:4])
        
        return sync_filter
    
    def DemodRTSignalsSet(self,demod_number,rt_signals):
        """
        Sets the signals available for acquisition on the real-time system from 
        the specified demodulator

        Parameters
        ----------
        demod_number : The number that specifies which demodulator to use. It 
                       starts from number 1 (=Demodulator 1)
        rt_signals : sets which signals from the specified demodulator should 
                     be available on the Real-time system. 
                     0 = sets the available 
                     RT Signals to X/Y
                     1 = sets them to R/phi

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.DemodRTSignalsSet', body_size=8)
        
        ## arguments
        hex_rep += self.NanonisTCP.to_hex(demod_number,4)
        hex_rep += self.NanonisTCP.to_hex(rt_signals,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
        
    def DemodRTSignalsGet(self,demod_number):
        """
        Returns which the signals are available for acquisition on the 
        real-time system from the specified demodulator.

        Parameters
        ----------
        demod_number : The number that specifies which demodulator to use. It 
                       starts from number 1 (=Demodulator 1)

        Returns
        -------
        rt_signals : Returns which signals from the specified demodulator 
                     should be available on the Real-time system. 
                     0 = X/Y
                     1 = R/phi

        """
        hex_rep = self.NanonisTCP.make_header('LockIn.DemodRTSignalsGet', body_size=4)
        
        hex_rep += self.NanonisTCP.to_hex(demod_number,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response()
        
        rt_signals = self.NanonisTCP.hex_to_uint32(response[0:4])
        
        return rt_signals


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Mar 29 11:25:57 2022

@author: Julian Ceddia and Jack Hellerstedt
"""

import struct
import socket


class nanonisTCP:
    def __init__(self, IP='127.0.0.1', PORT=6501, max_buf_size=200):
        """
        Parameters
        IP              : Listening IP address
        PORT            : Listening Port (check Nanonis File>Settings>TCP)
        max_buf_size    : maximum size of the response message. just make it big
        """
        self.IP   = IP
        self.PORT = PORT
        self.max_buf_size = max_buf_size
        
        self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)              # Set up the connection
        self.s.connect((IP, PORT))                                              # Open the TCP connection.

    def make_header(self, command_name, body_size, resp=True):
        """
        Parameters
        command_name : name of the Nanonis function
        body_size    : size of the message body in bytes
        resp         : tell nanonis to send a response. response contains error
                       message so will nearly always want to receive it

        Returns
        hex_rep : hex representation of the header string
        """
        hex_rep = command_name.encode('utf-8').hex()                            # command name
        hex_rep += "{0:#0{1}}".format(0,(64 - len(hex_rep)))                    # command name (fixed 32)
        hex_rep += self.to_hex(body_size, 4)                                    # Body size (fixed 4)
        hex_rep += self.to_hex(resp, 2)                                         # Send response (fixed 2)
        hex_rep += "{0:#0{1}}".format(0, 4)                                     # not used (fixed 2)
        return hex_rep
    
    ## Struct formatting:
    ## https://docs.python.org/3/library/struct.html#format-characters
    
    ## Hex to x conversions
    def hex_to_int16(self,h16):
        return struct.unpack("<h",struct.pack("H",int("0x"+h16.hex(),16)))[0]
    
    def hex_to_uint16(self,h16):
        return struct.unpack("<H",struct.pack("H",int("0x"+h16.hex(),16)))[0]
    
    def hex_to_int32(self,h32):
        return struct.unpack("<i",struct.pack("I",int("0x"+h32.hex(),16)))[0]
    
    def hex_to_uint32(self,h32):
        return struct.unpack("<I", struct.pack("I",int("0x"+h32.hex(), 16)))[0]
        
    def hex_to_float64(self,h64):
        # see https://forum.inductiveautomation.com/t/ieee-754-standard-converting-64-bit-hex-to-decimal/9324/3
        return struct.unpack("<d", struct.pack("Q",int("0x"+h64.hex(), 16)))[0]
    
    def hex_to_float32(self,h32):
        # see https://forum.inductiveautomation.com/t/ieee-754-standard-converting-64-bit-hex-to-decimal/9324/3
        return struct.unpack("<f", struct.pack("I",int("0x"+h32.hex(), 16)))[0]
    
    ## x to hex conversions
    def float64_to_hex(self,f64):
        # see https://stackoverflow.com/questions/23624212/how-to-convert-a-float-into-hex
        if(f64 == 0): return "0000000000000000"                                 # workaround for zero. look into this later
        return hex(struct.unpack('<Q', struct.pack('<d', f64))[0])[2:]          # float64 to hex
    
    def float32_to_hex(self,f32):
        # see https://stackoverflow.com/questions/23624212/how-to-convert-a-float-into-hex
        if(f32 == 0): return "00000000"                                         # workaround for zero. look into this later
        return hex(struct.unpack('<I', struct.pack('<f', f32))[0])[2:]          # float32 to hex
    
    def to_hex(self,conv,num_bytes):
        if(conv >= 0): return hex(conv)[2:].zfill(2*num_bytes)
        if(conv < 0):  return hex((conv + (1 << 8*num_bytes)) % (1 << 8*num_bytes))[2:]
    
    def string_to_hex(self,string):
        return string.encode('utf-8').hex()
    
    def send_command(self, message):
        """
        Parameters
        message : message to send over TCP to nanonis
        """
        self.s.send(bytes.fromhex(message))                                     # Convert hex into bytes object
        
    def receive_response(self, error_index=-1, keep_header = False):
        """
        Parameters
        error_index : index of 'error status' within the body. -1 skip check
        keep_header : if true: return entire response. if false: return body
        
        Returns
        response    : either header + body or body only (keep_header)
        
        """
        response = self.s.recv(self.max_buf_size)                               # Read the response
        body_size = self.hex_to_int32(response[32:36])
        while(True): 
            if(len(response) == body_size + 40): break                          # body_size + header size (40)
            response += self.s.recv(self.max_buf_size)
        
        if(error_index > -1): self.check_error(response[40:],error_index)       # error_index < 0 skips error check
        
        if(not keep_header):
            return response[40:]                                                # Header is fixed to 40 bytes - drop it
        
        return response
    
    def check_error(self,response,error_index):
        """
        Checks the response from nanonis for error messages

        Parameters
        response : response body (not inc. header) from nanonis (bytes)
        error_index : index of error status within the body

        Raises
        Exception   : error message returned from Nanonis

        """
        i = error_index                                                         # error_index points to start-byte in the body, which is after the 40-byte header
        error_status = self.hex_to_uint16(response[i:i+4])                      # error_status is 4 bytes long
        
        if(error_status):
            i += 8                                                              # index of error description is 8 bytes after error status
            error_description = response[i:].decode()                           # just grab from start index to the end of the message
            raise Exception(error_description)                                  # raise the exception
        
    
    def connect(self):
        """
        Open a once closed connection
        """
        self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)              # Set up the connection
        self.s.connect((self.IP, self.PORT))                                    # Open the TCP connection.
        
    def close_connection(self):
        """
        Close the TCP connection
        """
        self.s.close()


# -*- coding: utf-8 -*-
"""
Created on Tue Feb 2 18:49:58 2024

@author: jced0001
"""

class UserOut:
    """
    Nanonis User Outputs Module
    """
    def __init__(self, nanonisTCP):
        self.nanonisTCP = nanonisTCP
    
    def ModeSet(self, output_index, output_mode):
        """
        Sets the mode (User Output, Monitor, Calculated signal) of the selected user output channel.

        Parameters
        Output index (int) : Sets the output to be used, where index could be any value from 1 to the number of available outputs
        Output mode (unsigned int16) : Sets the output mode of the selected output, where 0=User Output, 1=Monitor, 2=Calc.Signal
        """
        ## Make Header
        hex_rep = self.nanonisTCP.make_header('UserOut.ModeSet', body_size=6)
        
        ## Arguments
        hex_rep += self.nanonisTCP.to_hex(output_index,4)
        hex_rep += self.nanonisTCP.to_hex(output_mode,2)
        
        self.nanonisTCP.send_command(hex_rep)
        
        self.nanonisTCP.receive_response(0)
    
    def ModeGet(self, output_index):
        """
        Returns the mode (User Output, Monitor, Calculated signal) of the selected user output channel.

        Parameters
        Output index (int) : Sets the output to be used, where index could be any value from 1 to the number of available outputs

        Returns
        Output mode (unsigned int16) : Returns the output mode of the selected output, where 0=User Output, 1=Monitor, 2=Calc.Signal, 3=Override

        """
        ## Make Header
        hex_rep = self.nanonisTCP.make_header('UserOut.ModeGet', body_size=4)

        ## Arguments
        hex_rep += self.nanonisTCP.to_hex(output_index,4)
        
        self.nanonisTCP.send_command(hex_rep)
        
        response = self.nanonisTCP.receive_response(2)
        
        output_mode = self.nanonisTCP.hex_to_uint16(response[0:2])
        return output_mode

    def MonitorChSet(self, output_index, monitor_channel_index):
        """
        Sets the monitor channel of the selected output channel.

        Parameters
        Output index (int) : Sets the output to be used, where index could be any value from 1 to the number of available outputs
        Monitor channel index (int) : Sets the index of the channel to monitor.
                                      The index is comprised between 0 and 127 for the physical inputs, physical outputs, and internal channels.
                                      To see which signal has which index, see Signals.NamesGet function.
        """
        ## Make Header
        hex_rep = self.nanonisTCP.make_header('UserOut.MonitorChSet', body_size=8)
        
        ## Arguments
        hex_rep += self.nanonisTCP.to_hex(output_index,4)
        hex_rep += self.nanonisTCP.to_hex(monitor_channel_index,4)
        
        self.nanonisTCP.send_command(hex_rep)
        
        self.nanonisTCP.receive_response(0)
        
    def MonitorChGet(self, output_index):
        """
        Returns the monitor channel of the selected output channel.

        Parameters
        Output index (int) : Sets the output to be used, where index could be any value from 1 to the number of available outputs

        Returns
        Monitor channel index (int): Returns the index of the channel to monitor.
                                     The index is comprised between 0 and 127 for the physical inputs, physical outputs, and internal channels.
                                     To see which signal has which index, see Signals.NamesGet function.

        """
        ## Make Header
        hex_rep = self.nanonisTCP.make_header('UserOut.MonitorChGet', body_size=4)

        ## Arguments
        hex_rep += self.nanonisTCP.to_hex(output_index,4)
        
        self.nanonisTCP.send_command(hex_rep)
        
        response = self.nanonisTCP.receive_response(4)
        
        monitor_channel_index = self.nanonisTCP.hex_to_int32(response[0:4])
        return monitor_channel_index
    
    def ValSet(self, output_index, output_value):
        """
        Sets the value of the selected user output channel.

        Parameters
        Output index (int) : Sets the output to be used, where index could be any value from 1 to the number of available outputs
        Output value (float32) : Is the value applied to the selected user output in physical units
        """
        ## Make Header
        hex_rep = self.nanonisTCP.make_header('UserOut.ValSet', body_size=8)
        
        ## Arguments
        hex_rep += self.nanonisTCP.to_hex(output_index,4)
        hex_rep += self.nanonisTCP.float32_to_hex(output_value)
        
        self.nanonisTCP.send_command(hex_rep)
        
        self.nanonisTCP.receive_response(0)
        
    def CalibrSet(self, output_index, calibration, offset):
        """
        Sets the calibration of the selected user output or monitor channel.

        Parameters
        Output index (int) : Sets the output to be used, where index could be any value from 1 to the number of available outputs
        Calibration per volt (float32)
        Offset in physical units (float32)
        """
        ## Make Header
        hex_rep = self.nanonisTCP.make_header('UserOut.CalibrSet', body_size=12)
        
        ## Arguments
        hex_rep += self.nanonisTCP.to_hex(output_index,4)
        hex_rep += self.nanonisTCP.float32_to_hex(calibration)
        hex_rep += self.nanonisTCP.float32_to_hex(offset)
        
        self.nanonisTCP.send_command(hex_rep)
        
        self.nanonisTCP.receive_response(0)

    def CalcSignalNameSet(self, output_index, calculated_signal_name):
        """
        Sets the Calculated Signal name of the selected output channel.

        Parameters
        Output index (int) : Sets the output to be used, where index could be any value from 1 to the number of available outputs
        Calculated signal name (string) is the name of the calculated signal configured for the selected output
        """
        calculated_signal_name_size = int(len(self.nanonisTCP.string_to_hex(calculated_signal_name))/2)
        body_size = 8 + calculated_signal_name_size
        
        ## Make Header
        hex_rep = self.nanonisTCP.make_header('UserOut.CalcSignalNameSet', body_size=body_size)
        
        ## Arguments
        hex_rep += self.nanonisTCP.to_hex(output_index,4)
        hex_rep += self.nanonisTCP.to_hex(calculated_signal_name_size,4)
        if(calculated_signal_name_size > 0):
            hex_rep += self.nanonisTCP.string_to_hex(calculated_signal_name)
        
        self.nanonisTCP.send_command(hex_rep)
        
        self.nanonisTCP.receive_response(0)

    
    def CalcSignalNameGet(self, output_index):
        """
        Returns the Calculated Signal name of the selected output channel.

        Parameters
        Output index (int) : Sets the output to be used, where index could be any value from 1 to the number of available outputs

        Returns
        Calculated signal name (string) is the name of the calculated signal configured for the selected output

        """
        ## Make Header
        hex_rep = self.nanonisTCP.make_header('UserOut.CalcSignalNameGet', body_size=4)

        ## Arguments
        hex_rep += self.nanonisTCP.to_hex(output_index,4)
        
        self.nanonisTCP.send_command(hex_rep)
        
        response = self.nanonisTCP.receive_response()
        
        calculated_signal_name = ""
        calculated_signal_name_size = self.nanonisTCP.hex_to_int32(response[0:4])
        if(calculated_signal_name_size > 0):
            calculated_signal_name = response[4:4+calculated_signal_name_size].decode()

        return calculated_signal_name
    
    def CalcSignalConfigSet(self, output_index, signal_1, operation, signal_2):
        """
        Sets the configuration of the Calculated Signal for the selected output channel
        The configuration is a math operation between 2 signals, or the logarithmic value of one signal.
        The possible values for the math operation are:
        0=None, 1=Add, 2=Subtract, 3=Multiply, 4=Divide, 6=Log

        Parameters
            Output index (int):          sets the output to be used, where index could be any value from 1 to the number of available outputs
            Signal 1 (unsigned int16):   is the signal index (from 0 to 127) used as the first signal of the formula.
            Operation (unsigned int16):  is the math operation.
            Signal 2 (unsigned int16):   is the signal index (from 0 to 127) used as the second signal of the formula.
        """
        ## Make Header
        hex_rep = self.nanonisTCP.make_header('UserOut.CalcSignalConfigSet', body_size=10)
        
        ## Arguments
        hex_rep += self.nanonisTCP.to_hex(output_index,4)
        hex_rep += self.nanonisTCP.to_hex(signal_1,2)
        hex_rep += self.nanonisTCP.to_hex(operation,2)
        hex_rep += self.nanonisTCP.to_hex(signal_2,2)
        
        self.nanonisTCP.send_command(hex_rep)
        
        self.nanonisTCP.receive_response(0)
        
    def CalcSignalConfigGet(self, output_index):
        """
        Returns the configuration of the Calculated Signal for the selected output channel.
        The configuration is a math operation between 2 signals, or the logarithmic value of one signal.
        The possible values for the math operation are:
        0=None, 1=Add, 2=Subtract, 3=Multiply, 4=Divide, 6=Log

        Parameters
            Output index (int) : Sets the output to be used, where index could be any value from 1 to the number of available outputs

        Returns
            Signal 1 (unsigned int16):   is the signal index (from 0 to 127) used as the first signal of the formula.
            Operation (unsigned int16):  is the math operation.
            Signal 2 (unsigned int16):   is the signal index (from 0 to 127) used as the second signal of the formula.

        """
        ## Make Header
        hex_rep = self.nanonisTCP.make_header('UserOut.CalcSignalConfigGet', body_size=4)

        ## Arguments
        hex_rep += self.nanonisTCP.to_hex(output_index,4)
        
        self.nanonisTCP.send_command(hex_rep)
        
        response = self.nanonisTCP.receive_response()
        
        signal_1  = self.nanonisTCP.hex_to_uint16(response[0:2])
        operation = self.nanonisTCP.hex_to_uint16(response[2:4])
        signal_2  = self.nanonisTCP.hex_to_uint16(response[4:6])

        return [signal_1, operation, signal_2]
    
    def LimitsSet(self, output_index, upper_limit, lower_limit):
        """
        Sets the physical limits (in calibrated units) of the selected output channel.

        Parameters
            Output index (int):    Sets the output to be used, where index could be any value from 1 to the number of available outputs
            Upper limit (float32): Defines the upper physical limit of the user output
            Lower limit (float32): Defines the lower physical limit of the user output
        """
        ## Make Header
        hex_rep = self.nanonisTCP.make_header('UserOut.LimitsSet', body_size=12)
        
        ## Arguments
        hex_rep += self.nanonisTCP.to_hex(output_index,4)
        hex_rep += self.nanonisTCP.float32_to_hex(upper_limit)
        hex_rep += self.nanonisTCP.float32_to_hex(lower_limit)
        
        self.nanonisTCP.send_command(hex_rep)
        
        self.nanonisTCP.receive_response(0)
        
    def LimitsGet(self, output_index):
        """
        Returns the physical limits (in calibrated units) of the selected output channel.

        Parameters
            Output index (int) : Sets the output to be used, where index could be any value from 1 to the number of available outputs

        Returns
            Upper limit (float32): Defines the upper physical limit of the user output
            Lower limit (float32): Defines the lower physical limit of the user output

        """
        ## Make Header
        hex_rep = self.nanonisTCP.make_header('UserOut.LimitsGet', body_size=4)

        ## Arguments
        hex_rep += self.nanonisTCP.to_hex(output_index,4)
        
        self.nanonisTCP.send_command(hex_rep)
        
        response = self.nanonisTCP.receive_response()
        
        lower_limit = self.nanonisTCP.hex_to_float32(response[0:4])
        upper_limit = self.nanonisTCP.hex_to_float32(response[4:8])

        return [lower_limit, upper_limit]


# -*- coding: utf-8 -*-
"""
Created on Tue May  3 19:09:44 2022

@author: Julian Ceddia
"""
"""
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!WARNING: RUNNING run_test() WILL AUTO APPROACH YOUR TIP. RUN IT ON A SIM!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
"""


from nanonisTCP import nanonisTCP
from nanonisTCP.AutoApproach import AutoApproach

"""
Set up the TCP connection and interface
"""
def run_test(TCP_IP='127.0.0.1', TCP_PORT=6502, make_plot=False):
    
    # Listening Port: see Nanonis File>Settings>TCP Programming Interface
    NTCP = nanonisTCP(TCP_IP, TCP_PORT)                                         # Nanonis TCP interface
    
    autoApp = AutoApproach(NTCP)                                                # Nanonis Auto Approach Module
    
    try:
        """
        Open
        """
        autoApp.Open()
        
        """
        On/Off Set/Get
        """
        autoApp.OnOffSet(on_off=True)
        on_off = autoApp.OnOffGet()
        print("Auto Approach On/Off")
        print("Status: " + ["Off","Running"][on_off])
        print("----------------------------------------------------------------------")
    except Exception as e:
        print(e)
    
    NTCP.close_connection()


# -*- coding: utf-8 -*-
"""
Created on Tue Feb 2 18:56:49 2024

@author: jced0001
"""

from nanonisTCP import nanonisTCP
from nanonisTCP.UserOut import UserOut

"""
Set up the TCP connection and interface
"""
def run_test(TCP_IP='127.0.0.1', TCP_PORT=6501):
    # Listening Port: see Nanonis File>Settings>TCP Programming Interface
    NTCP = nanonisTCP(TCP_IP, TCP_PORT)                                             # Nanonis TCP interface
    try:
        userOut = UserOut(NTCP)                                                         # Nanonis User Outs Module

        """
        Output Mode Set/Get
        """
        userOut.ModeSet(2,1)                                                            # Set Out2 to Monitor
        output_mode = userOut.ModeGet(2)                                                # Check to see if the mode for Output2 has changed
        print("UserOut ModeGet")
        print("Output Mode: ",["User Output","Monitor","Calc.Signal"][output_mode])
        print("----------------------------------------------------------------------")

        """
        Output Monitor Set/Get
        """
        userOut.MonitorChSet(2,1)                                                       # Set the monitor channel to index 1 for output 2
        monitor_index = userOut.MonitorChGet(2)                                         # Check to see if it changed.
        print("UserOut MonitorChGet")
        print("Monitor index: ",monitor_index)
        print("----------------------------------------------------------------------")

        """
        Output Val Set
        (There is no Get in the protocol)
        """
        userOut.ValSet(8,3.3)
        
        """
        Output Calibration Set
        (There is no Get in the protocol)
        """
        userOut.CalibrSet(8,0.2,0.1)

        """
        Output CalcSignalName Set/Get
        """
        userOut.CalcSignalNameSet(8,"Output 8 Test")
        calculated_signal_name = userOut.CalcSignalNameGet(8)
        print("UserOut CalcSignalNameGet")
        print("Signal name: ",calculated_signal_name)
        print("----------------------------------------------------------------------")

        """
        Output CalcSignalConfig Set/Get
        """
        userOut.CalcSignalConfigSet(8,1,2,1)
        signal_1,operation,signal_2 = userOut.CalcSignalConfigGet(8)
        print("UserOut CalcSignalConfiGet")
        print("Signal 1:  ",signal_1)
        print("Operation: ",operation)
        print("Signal 2:  ",signal_2)
        print("----------------------------------------------------------------------")

        """
        Output Limits Get/Set
        """
        userOut.LimitsSet(8,-3.3,4.3)
        lower_limit,upper_limit = userOut.LimitsGet(8)
        print("UserOut Limits Get")
        print("Lower Limit:",lower_limit)
        print("Upper Limit:",upper_limit)
        print("----------------------------------------------------------------------")
    except Exception as e:
        print('error',e)
    NTCP.close_connection()
    return print('end of test')

run_test()


# -*- coding: utf-8 -*-
"""
Created on Tue May  3 15:17:56 2022

@author: Julian Ceddia
"""
"""
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!WARNING: RUNNING run_test() WILL CHANGE SETTINGS IN NANONIS. RUN IT ON A SIM!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
"""

from nanonisTCP import nanonisTCP
from nanonisTCP.SafeTip import SafeTip

"""
Set up the TCP connection and interface
"""
def run_test(TCP_IP='127.0.0.1', TCP_PORT=6502, make_plot=False):
    
    # Listening Port: see Nanonis File>Settings>TCP Programming Interface
    NTCP = nanonisTCP(TCP_IP, TCP_PORT)                                         # Nanonis TCP interface
    
    safeTip = SafeTip(NTCP)                                                     # Nanonis Safe Tip Module
    
    try:
        """
        On/Off Set/Get
        """
        safeTip.OnOffSet(safe_tip_status=2)
        safe_tip_status = safeTip.OnOffGet()
        print("Safe Tip Status")
        print("Safe Tip Mode:  " + ["off","on"][safe_tip_status])
        print("----------------------------------------------------------------------")
        
        """
        Signal Get
        """
        signal_value = safeTip.SignalGet()
        print("Safe Tip Signal")
        print("Signal value:  " + str(signal_value) + " A")
        print("----------------------------------------------------------------------")
        
        """
        Properties Set/Get
        """
        safeTip.PropsSet(auto_recovery=True,auto_pause_scan=True,threshold=15e-10)
        auto_recovery,auto_pause_scan,threshold = safeTip.PropsGet()
        print("Safe Tip Properties")
        print("Auto recovery:  " + ["Off","On"][auto_recovery])
        print("Auto pause:     " + ["Off","On"][auto_pause_scan])
        print("Threshold:      " + str(threshold) + " A")
        print("----------------------------------------------------------------------")
    except Exception as e:
        print(e)
    
    NTCP.close_connection()


# -*- coding: utf-8 -*-
"""
Created on Wed Aug 24 18:33:28 2022

@author: Julian
"""

import numpy as np

class BiasSpectr:
    """
    Nanonis Bias Spectroscopy Module
    """
    def __init__(self,NanonisTCP):
        self.NanonisTCP = NanonisTCP
        
    def Open(self):
        """
        Opens the Bias Spectroscopy Module

        """
        ## Make Header
        hex_rep = self.NanonisTCP.make_header('BiasSpectr.Open', body_size=0)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
    
    def Start(self,get_data,save_base_name=""):
        """
        Starts a bias spectroscopy in the Bias Spectroscopy module.
        
        Before using this function, select the channels to record in the Bias
        Spectroscopy module.

        Parameters
        ----------
        get_data        : defines if the function returns the spectroscopy data
                          True: return data from this function
                          False: don't return data
        save_base_name  : Base name used by the saved files. Empty string 
                          keeps settings unchanged in nanonis
                    

        Returns
        -------
        if get_data  = False, this function returns None
        
        if get_data != False, this function returns:
            
        data_dict{
            '<channel_name>' : data for this channel
            }
        parameters  : List of fixed parameters and parameters (in that order).
                      To see the names of the returned parameters, use the 
                      BiasSpectr.PropsGet function

        """
        body_size = 4 + 4                                                       # 4 bytes for get_data (uint32) and 4 bytes for save_base_name_string_size (int)
        body_size += int(len(self.NanonisTCP.string_to_hex(save_base_name))/2)   # Variable size depending on the save_base_name string
        
        ## Make Header
        hex_rep = self.NanonisTCP.make_header('BiasSpectr.Start', body_size=body_size)
        
        save_base_name_string_size = len(save_base_name)
        
        ## arguments
        hex_rep += self.NanonisTCP.to_hex(get_data,4)
        hex_rep += self.NanonisTCP.to_hex(save_base_name_string_size,4)
        if(save_base_name_string_size > 0):
            hex_rep += self.NanonisTCP.string_to_hex(save_base_name)
        
        self.NanonisTCP.send_command(hex_rep)
        
        if(not get_data == 1):
            response = self.NanonisTCP.receive_response(0)
            return
        
        # Receive Response
        response = self.NanonisTCP.receive_response()
        
        # channels_names_size = self.NanonisTCP.hex_to_int32(response[0:4])     # Useless
        number_of_channels  = self.NanonisTCP.hex_to_int32(response[4:8])
        
        idx = 8
        channel_names = []
        for i in range(number_of_channels):
            channel_name_size = self.NanonisTCP.hex_to_int32(response[idx:idx+4])
            idx += 4
            channel_names.append(response[idx:idx + channel_name_size].decode())
            idx += channel_name_size
        
        data_rows = self.NanonisTCP.hex_to_int32(response[idx:idx+4])
        idx += 4
        data_cols = self.NanonisTCP.hex_to_int32(response[idx:idx+4])
        
        data_dict = {}
        for i in range(data_rows):
            data = []
            for j in range(data_cols):
                idx += 4
                data.append(self.NanonisTCP.hex_to_float32(response[idx:idx+4]))
            data_dict[channel_names[i]] = np.array(data)
        
        idx += 4
        parameters = []
        number_of_parameters = self.NanonisTCP.hex_to_int32(response[idx:idx+4])
        for i in range(number_of_parameters):
            idx += 4
            parameter = self.NanonisTCP.hex_to_float32(response[idx:idx+4])
            parameters.append(parameter)
        
        return {"data_dict"  : data_dict,
                "parameters" : parameters}
    
    def Stop(self):
        """
        Stops the current Bias Spectroscopy measurement.

        """
        hex_rep = self.NanonisTCP.make_header('BiasSpectr.Stop', body_size=0)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
    
    def StatusGet(self):
        """
        Returns the status of the Bias Spectroscopy measurement.
        
        Returns
        -------
        status : 0: Not running. 1: Running

        """
        hex_rep = self.NanonisTCP.make_header('BiasSpectr.StatusGet', body_size=0)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response(4)
        
        status = self.NanonisTCP.hex_to_int32(response[0:4])
        
        return status
        
    def ChsSet(self,channel_indexes,mode="set"):
        """
        Sets/adds/removes the list of recortded channels in Bias Spectroscopy

        Parameters
        ----------
        channel_indexes : channel indexes to set, add, or remove. The indexes 
                          are comprised between 0 and 23 for the 24 signals 
                          assigned in the Signals Manager. To get the signal 
                          name and its corresponding index in the list of 128 
                          available signals in the Nanonis Controller, use 
                          the Signals.InSlotsGet function
        mode            : "set"    : channel_indexes will become the entire 
                                     list of selected channels
                          "add"    : channel_indexes will be added to the list 
                                     of selected channels
                          "remove" : remove channel_indexes from list of 
                                     selected channels

        """
        if(mode not in ["set","add","remove"]):
            raise Exception("Invalid mode for BiasSpectr.ChsSet. Must be one" +
                            "of 'set', 'add', 'remove'. Got " + str(mode))
        
        if(mode == "add"):
            buf_channel_indexes = self.ChsGet()
            for buf_index in buf_channel_indexes:
                if(buf_index not in channel_indexes):
                    channel_indexes.append(buf_index)
            
        if(mode == "remove"):
            buf_channel_indexes = self.ChsGet()
            for index in channel_indexes:
                if(index in buf_channel_indexes):
                    buf_channel_indexes.remove(index)
            channel_indexes = buf_channel_indexes
            
        number_of_channels = len(channel_indexes)
        body_size = 4 + number_of_channels*4                                    # 4 bytes for number_of_channels (int), plus 4 bytes for the integer array of channel_indexes
        hex_rep = self.NanonisTCP.make_header('BiasSpectr.ChsSet', body_size=body_size)
        
        ## arguments
        hex_rep += self.NanonisTCP.to_hex(number_of_channels,4)
        for index in channel_indexes:
            hex_rep += self.NanonisTCP.to_hex(index,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
        
    def ChsGet(self):
        """
        Returns the list of recorded channels in Bias Spectroscopy

        Returns
        -------
        channel_indexes : The indexes of recorded channels. The indexes are 
                          comprised between 0 and 23 for the 24 signals 
                          assigned in the signals manager.
                          To get the signal name and its corresponding index in
                          the list of 128 available signals in the Nanonis
                          Controller, use the Signals.InSlotsGet function

        """
        hex_rep = self.NanonisTCP.make_header('BiasSpectr.ChsGet', body_size=0)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response()
        
        number_of_channels = self.NanonisTCP.hex_to_int32(response[0:4])
        
        idx = 4
        channel_indexes = []
        for i in range(number_of_channels):
            channel_index = self.NanonisTCP.hex_to_int32(response[idx:idx+4])
            channel_indexes.append(channel_index)
            idx += 4
        
        return channel_indexes
        
    def PropsSet(self,save_all=0,num_sweeps=0,back_sweep=0,num_points=0,z_offset=0,autosave=0,save_dialog=0):
        """
        Configures the Bias Spectroscopy parameters

        Returns
        -------
        save_all : 0 : no change
                   1 : data from individual sweeps is saved, along with 
                       averaged data.
                   2 : Individual sweeps are not saved, only the average of 
                       all of them is saved. This parameter only makes sense 
                       when multiple sweeps are configured.
        num_sweeps : Number of sweeps to measure and average. 0 means no change
        back_sweep : Selects whether a backward sweep is acquired in addition 
                     to the forward sweep (which is always acquired)
                     0 : No change
                     1 : Don't acquire a backward sweep.
                     2 : Acquire a backward sweep (in addition to the forward)
        num_points : Defines the number of points to acquire over the sweep
                     range. 0 means no change
        z_offset   : Defines which distance (m) to move the tip before starting
                     the measurement. Positive value means retracting,
                     negative value means approaching.
        autosave   : Selects whether to automatically save the data to ASCII 
                     file once the sweep is done.
                     0 : No change
                     1 : Autosave data after acquisition
                     2 : Don't autosave after acquisition
        save_dialog : Selects whether to show the save dialog box once the 
                      sweep is done.
                      0 : No change
                      1 : Show the save dialog box after sweep
                      2 : Don't show the save dialog box after sweep

        """
        hex_rep = self.NanonisTCP.make_header('BiasSpectr.PropsSet', body_size=20)
        
        ## arguments
        hex_rep += self.NanonisTCP.to_hex(save_all,2)
        hex_rep += self.NanonisTCP.to_hex(num_sweeps,4)
        hex_rep += self.NanonisTCP.to_hex(back_sweep,2)
        hex_rep += self.NanonisTCP.to_hex(num_points,4)
        hex_rep += self.NanonisTCP.float32_to_hex(z_offset)
        hex_rep += self.NanonisTCP.to_hex(autosave,2)
        hex_rep += self.NanonisTCP.to_hex(save_dialog,2)
        
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
        
    def PropsGet(self):
        """
        Returns the Bias Spectroscopy parameters

        Returns
        -------
        save_all : 1 : data from individual sweeps is saved, along with 
                       averaged data.
                   0 : Individual sweeps are not saved, only the average of 
                       all of them is saved. This parameter only makes sense 
                       when multiple sweeps are configured.
        num_sweeps : Number of sweeps to measure and average. 0 means no change
        back_sweep : Selects whether a backward sweep is acquired in addition 
                     to the forward sweep (which is always acquired)
                     0 : Don't acquire a backward sweep.
                     1 : Acquire a backward sweep (in addition to the forward)
        num_points : Defines the number of points to acquire over the sweep
                     range. 0 means no change
        channels   : Returns the names of the acquired channels in the sweep.
        parameters : Returns the parameters of the sweep
        fixed_parameters : Returns the fixed parameters of the sweep
        
        """
        hex_rep = self.NanonisTCP.make_header('BiasSpectr.PropsGet', body_size=0)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response()
        
        save_all   = self.NanonisTCP.hex_to_int16(response[0:2])
        num_sweeps = self.NanonisTCP.hex_to_int32(response[2:6])
        back_sweep = self.NanonisTCP.hex_to_int16(response[6:8])
        num_points = self.NanonisTCP.hex_to_int32(response[8:12])
        
        # channels_size = self.NanonisTCP.hex_to_int32(response[12:16])         # Useless
        idx = 20
        channels = []
        num_channels = self.NanonisTCP.hex_to_int32(response[16:20])
        for channel in range(num_channels):
            channel_size = self.NanonisTCP.hex_to_int32(response[idx:idx+4])
            idx += 4
            channels.append(response[idx:idx+channel_size].decode())
            idx += channel_size
            
        # parameters_size = self.NanonisTCP.hex_to_int32(response[idx:idx+4])   # Useless
        
        idx += 4
        parameters = []
        num_parameters  = self.NanonisTCP.hex_to_int32(response[idx:idx+4])
        idx += 4
        for parameter in range(num_parameters):
            parameter_size = self.NanonisTCP.hex_to_int32(response[idx:idx+4])
            idx += 4
            parameters.append(response[idx:idx+parameter_size].decode())
            idx += parameter_size
            
        # fixed_parameters_size = self.NanonisTCP.hex_to_int32(response[idx:idx+4]) # Useless
        idx += 4
        fixed_parameters = []
        num_fixed_parameters  = self.NanonisTCP.hex_to_int32(response[idx:idx+4])
        idx += 4
        for fixed_parameter in range(num_fixed_parameters):
            fixed_parameter_size = self.NanonisTCP.hex_to_int32(response[idx:idx+4])
            idx += 4
            fixed_parameters.append(response[idx:idx+fixed_parameter_size].decode())
            idx += fixed_parameter_size
        
        return {"save_all"    : save_all,
                "num_sweeps"  : num_sweeps,
                "back_sweep"  : back_sweep,
                "num_points"  : num_points,
                "channels"    : channels,
                "parameters"  : parameters,
                "fixed_parameters" : fixed_parameters}
    
    def AdvPropsSet(self,reset_bias=0,z_controller_hold=0,record_final_z=0,lockin_run=0):
        """
        Sets parameters from the advanced configuration section of the bias 
        spectroscopy module.

        Parameters
        ----------
        reset_bias          : Sets whether the Bias voltage returns to the 
                              initial value at the end of the spectroscopy 
                              measurement.
                              0: no change
                              1: on
                              2: off
        z_controller_hold   : Sets the Z-Controller on hold during the sweeo.
                              0: no change
                              1: on
                              2: off
        record_final_z      : Records the Z position during the Z averaging 
                              time at the end of the sweep.
                              0: no change
                              1: on
                              2: off
        lockin_run          : Sets the lock-in to run during the measurement
                              0: no change
                              1: on
                              2: off

        """
        hex_rep = self.NanonisTCP.make_header('BiasSpectr.AdvPropsSet', body_size=8)
        
        ## arguments
        hex_rep += self.NanonisTCP.to_hex(reset_bias,2)
        hex_rep += self.NanonisTCP.to_hex(z_controller_hold,2)
        hex_rep += self.NanonisTCP.to_hex(record_final_z,2)
        hex_rep += self.NanonisTCP.to_hex(lockin_run,2)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
    
    def AdvPropsGet(self):
        """
        Returns parameters from the advanced configuration section of the bias 
        spectroscopy module.

        Parameters
        ----------
        reset_bias          : Sets whether the Bias voltage returns to the 
                              initial value at the end of the spectroscopy 
                              measurement.
                              1: on
                              0: off
        z_controller_hold   : Sets the Z-Controller on hold during the sweeo.
                              1: on
                              0: off
        record_final_z      : Records the Z position during the Z averaging 
                              time at the end of the sweep.
                              1: on
                              0: off
        lockin_run          : Sets the lock-in to run during the measurement
                              1: on
                              0: off

        """
        hex_rep = self.NanonisTCP.make_header('BiasSpectr.AdvPropsGet', body_size=0)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response()
        
        reset_bias          = self.NanonisTCP.hex_to_int16(response[0:2])
        z_controller_hold   = self.NanonisTCP.hex_to_int16(response[2:4])
        record_final_z      = self.NanonisTCP.hex_to_int16(response[4:6])
        lockin_run          = self.NanonisTCP.hex_to_int16(response[6:8])
        
        return {"reset_bias"        : reset_bias,
                "z_controller_hold" : z_controller_hold,
                "record_final_z"    : record_final_z,
                "lockin_run"        : lockin_run}
    
    def LimitsSet(self,start_value,end_value):
        """
        Sets the Bias spectroscopy limits.

        Parameters
        ----------
        start_value : is the starting value of the sweep
        end_value   : is the ending value of the sweep
        
        """
        hex_rep = self.NanonisTCP.make_header('BiasSpectr.LimitsSet', body_size=8)
        
        ## arguments
        hex_rep += self.NanonisTCP.float32_to_hex(start_value)
        hex_rep += self.NanonisTCP.float32_to_hex(end_value)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
    
    def LimitsGet(self):
        """
        Returns the bias spectroscopy limits

        Returns
        -------
        start_value : is the starting value of the sweep
        end_value   : is the ending value of the sweep

        """
        hex_rep = self.NanonisTCP.make_header('BiasSpectr.LimitsGet', body_size=0)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response()
        
        start_value = self.NanonisTCP.hex_to_float32(response[0:4])
        end_value   = self.NanonisTCP.hex_to_float32(response[4:8])
        
        return {"start_value" : start_value,
                "end_value"   : end_value}
        
    def TimingSet(self,z_averaging_time,z_offset,initial_settling_time,maximum_slew_rate,settling_time,integration_time,end_settling_time,z_control_time):
        """
        Configures the bias spec timing parameters

        Parameters
        ----------
        z_averaging_time :      
        z_offset :              
        initial_settling_time : 
        maximum_slew_rate :     
        settling_time :         
        integration_time :      
        end_settling_time :     
        z_control_time :        

        """
        hex_rep = self.NanonisTCP.make_header('BiasSpectr.TimingSet', body_size=32)
        
        ## arguments
        hex_rep += self.NanonisTCP.float32_to_hex(z_averaging_time)
        hex_rep += self.NanonisTCP.float32_to_hex(z_offset)
        hex_rep += self.NanonisTCP.float32_to_hex(initial_settling_time)
        hex_rep += self.NanonisTCP.float32_to_hex(maximum_slew_rate)
        hex_rep += self.NanonisTCP.float32_to_hex(settling_time)
        hex_rep += self.NanonisTCP.float32_to_hex(integration_time)
        hex_rep += self.NanonisTCP.float32_to_hex(end_settling_time)
        hex_rep += self.NanonisTCP.float32_to_hex(z_control_time)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
        
    def TimingGet(self):
        """
        Returns the bias spec timing params

        Returns
        -------
        z_averaging_time :      
        z_offset :              
        initial_settling_time : 
        maximum_slew_rate :     
        settling_time :         
        integration_time :      
        end_settling_time :     
        z_control_time :        

        """
        hex_rep = self.NanonisTCP.make_header('BiasSpectr.TimingGet', body_size=0)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response()
        
        z_averaging_time        = self.NanonisTCP.hex_to_float32(response[0:4])
        z_offset                = self.NanonisTCP.hex_to_float32(response[4:8])
        initial_settling_time   = self.NanonisTCP.hex_to_float32(response[8:12])
        maximum_slew_rate       = self.NanonisTCP.hex_to_float32(response[12:16])
        settling_time           = self.NanonisTCP.hex_to_float32(response[16:20])
        integration_time        = self.NanonisTCP.hex_to_float32(response[20:24])
        end_settling_time       = self.NanonisTCP.hex_to_float32(response[24:28])
        z_control_time          = self.NanonisTCP.hex_to_float32(response[28:32])
        
        return {"z_averaging_time"      : z_averaging_time,
                "z_offset"              : z_offset,
                "initial_settling_time" : initial_settling_time,
                "maximum_slew_rate"     : maximum_slew_rate,
                "settling_time"         : settling_time,
                "integration_time"      : integration_time,
                "end_settling_time"     : end_settling_time,
                "z_control_time"        : z_control_time}
    
    # def TTLSyncSet(self)
    # def TTLSyncGet(self)
    
    def AltZCtrlSet(self,alternate_setpoint_onoff,setpoint,settling_time):
        """
        Sets the configuration of the alternate z-controller setpoint in the
        advanced section of the bias spec module.
        
        When switched on, the z-controller setpoint is set to <setpoint> 
        right after starting the measuremnt. After changing the setpoint, the 
        settling time (s) will be waited for the z-controller to adjust to the 
        modified setpoint.
        
        Then the z averaging will start. The original z-controller setpoint is 
        restored at the end of the measurement, before restoring the
        z-controller state.

        Parameters
        ----------
        alternate_setpoint_onoff :  Turn the alternate z-controller setpoint on
                                    0: no change
                                    1: on
                                    2: off
        setpoint :                  setpoint for spectroscopy (A)
        settling_time :             Settling time (s)

        """
        hex_rep = self.NanonisTCP.make_header('BiasSpectr.AltZCtrlSet', body_size=10)
        
        ## arguments
        hex_rep += self.NanonisTCP.to_hex(alternate_setpoint_onoff,2)
        hex_rep += self.NanonisTCP.float32_to_hex(setpoint)
        hex_rep += self.NanonisTCP.float32_to_hex(settling_time)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
        
    def AltZCtrlGet(self):
        """
        Returns the configuration of the alternate z-controller setpoint in the
        advanced section of the bias spec module

        Returns
        -------
        alternate_setpoint_onoff :  Turn the alternate z-controller setpoint on
                                    0: off
                                    1: on
        setpoint :                  setpoint for spectroscopy (A)
        settling_time :             Settling time (s)

        """
        hex_rep = self.NanonisTCP.make_header('BiasSpectr.AltZCtrlGet', body_size=0)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response()
        
        alternate_setpoint_onoff = self.NanonisTCP.hex_to_uint16(response[0:2])
        setpoint                 = self.NanonisTCP.hex_to_float32(response[2:6])
        settling_time            = self.NanonisTCP.hex_to_float32(response[6:10])
        
        return {"alternate_setpoint_onoff"  : alternate_setpoint_onoff,
                "setpoint"                  : setpoint,
                "settling_time"             : settling_time}
    
    def MLSLockinPerSegSet(self,lockin_per_segment):
        """
        Sets the Lock-In Per Segment flag in the multi line segment editor
        
        When selected, the Lock-In can be defined per segment in the Multi line 
        segment editor. Otherwise, the Lock-In is set globally according to the 
        flag in the Advanced section of Bias spectroscopy

        Parameters
        ----------
        lockin_per_segment : 0: off; 1: on

        """
        hex_rep = self.NanonisTCP.make_header('BiasSpectr.MLSLockinPerSegSet', body_size=4)
        
        ## arguments
        hex_rep += self.NanonisTCP.to_hex(lockin_per_segment,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
        
    def MLSLockinPerSegGet(self):
        """
        Returns the Lock-In per Segment flag in the Multi line segment editor.
        When selected, the Lock-In can be defined per segment in the Multi line
        segment editor. Otherwise, the Lock-In is set globally according to the 
        flag in the Advanced section of Bias spectroscopy

        Returns
        -------
        lockin_per_segment : 0: off; 1: on

        """
        hex_rep = self.NanonisTCP.make_header('BiasSpectr.MLSLockinPerSegGet', body_size=0)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response()
        
        lockin_per_segment = self.NanonisTCP.hex_to_uint32(response[0:4])
        
        return lockin_per_segment
    
    def MLSModeSet(self,sweep_mode):
        """
        Sets the Bias Spectroscopy sweep mode.

        Parameters
        ----------
        sweep_mode : "linear" for linear or "MLS" for MultiSegment mode

        """
        if(sweep_mode.lower() == "linear"): sweep_mode = "Linear"
        if(sweep_mode.lower() == "mls"):    sweep_mode = "MLS"
        sweep_mode_size = len(sweep_mode)
        
        body_size = 4 + int(len(self.NanonisTCP.string_to_hex(sweep_mode))/2)
        hex_rep = self.NanonisTCP.make_header('BiasSpectr.MLSModeSet', body_size=body_size)
        
        ## arguments
        hex_rep += self.NanonisTCP.to_hex(sweep_mode_size,4)
        hex_rep += self.NanonisTCP.string_to_hex(sweep_mode)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
    
    def MLSModeGet(self):
        """
        Returns the Bias Spectroscopy sweep mode.

        Returns
        -------
        sweep_mode : "linear" for linear or "MLS" for MultiSegment mode

        """
        hex_rep = self.NanonisTCP.make_header('BiasSpectr.MLSModeGet', body_size=0)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response()
        
        sweep_mode = response[4:].decode()
        
        return sweep_mode
    
    def MLSValsSet(self,bias_start,bias_end,initial_settling_time,settling_time,integration_time,steps,lockin_run):
        """
        Sets the bias spectroscopy multiple line segment configuration for 
        Multi Line Segment mode. Up to 16 distinct line segments may be 
        defined. Any segments beyond the maximum allowed amount will be ignored
        
        Parameters (all type list)
        ----------
        bias_start              : list of bias start values for each segment (V)
        bias_end                : list of bias end values for each segment (V)
        initial_settling_time   : time to wait at begining of each segment
                                  before Lock-In setting is applied (s)
        settling_time           : time to wait before measuring each data point
                                  for each segment (s)
        integration_time        : integration time for each data point for 
                                  each segment (s)
        steps                   : number of steps to measure in each segment
        lockin_run              : indicates if the Lock-In will run during the 
                                  segment. This is true only if the global 
                                  Lock-In per Segment flag is enabled.
                                  Otherwise, the Lock-In is set globally 
                                  according to the flag in the Advanced section 
                                  of Bias spectroscopy
        """
        if(not type(bias_start) is list): bias_start = [bias_start]
        if(not type(bias_end) is list): bias_end = [bias_end]
        if(not type(initial_settling_time) is list): initial_settling_time = [initial_settling_time]
        if(not type(settling_time) is list): settling_time = [settling_time]
        if(not type(integration_time) is list): integration_time = [integration_time]
        if(not type(steps) is list): steps = [steps]
        if(not type(lockin_run) is list): lockin_run = [lockin_run]
        
        num_segments = len(bias_start)
        
        body_size = 4 + num_segments*(4 + 4 + 4 + 4 + 4 + 4 + 4)
        hex_rep = self.NanonisTCP.make_header('BiasSpectr.MLSValsSet', body_size=body_size)
        
        hex_rep += self.NanonisTCP.to_hex(num_segments,4)
        
        for p in bias_start:
            hex_rep += self.NanonisTCP.float32_to_hex(p)
            
        for p in bias_end:
            hex_rep += self.NanonisTCP.float32_to_hex(p)
            
        for p in initial_settling_time:
            hex_rep += self.NanonisTCP.float32_to_hex(p)
            
        for p in settling_time:
            hex_rep += self.NanonisTCP.float32_to_hex(p)
            
        for p in integration_time:
            hex_rep += self.NanonisTCP.float32_to_hex(p)
            
        for p in steps:
            hex_rep += self.NanonisTCP.to_hex(p,4)
            
        for p in lockin_run:
            hex_rep += self.NanonisTCP.to_hex(p,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
    
    def MLSValsGet(self):
        """
        Returns the bias spectroscopy multiple line segment configuration for 
        Multi Line Segment mode. Up to 16 distinct line segments may be defined.

        Returns
        -------
        bias_start              : list of bias start values for each segment (V)
        bias_end                : list of bias end values for each segment (V)
        initial_settling_time   : time to wait at begining of each segment
                                  before Lock-In setting is applied (s)
        settling_time           : time to wait before measuring each data point
                                  for each segment (s)
        integration_time        : integration time for each data point for 
                                  each segment (s)
        steps                   : number of steps to measure in each segment
        lockin_run              : indicates if the Lock-In will run during the 
                                  segment. This is true only if the global 
                                  Lock-In per Segment flag is enabled.
                                  Otherwise, the Lock-In is set globally 
                                  according to the flag in the Advanced section 
                                  of Bias spectroscopy

        """
        hex_rep = self.NanonisTCP.make_header('BiasSpectr.MLSValsGet', body_size=0)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response()
        
        num_segments = self.NanonisTCP.hex_to_int32(response[0:4])
        
        idx = 4
        bias_start = []
        for p in range(num_segments):
            bias_start.append(self.NanonisTCP.hex_to_float32(response[idx:idx+4]))
            idx += 4
            
        bias_end = []
        for p in range(num_segments):
            bias_end.append(self.NanonisTCP.hex_to_float32(response[idx:idx+4]))
            idx += 4
            
        initial_settling_time = []
        for p in range(num_segments):
            initial_settling_time.append(self.NanonisTCP.hex_to_float32(response[idx:idx+4]))
            idx += 4
            
        settling_time = []
        for p in range(num_segments):
            settling_time.append(self.NanonisTCP.hex_to_float32(response[idx:idx+4]))
            idx += 4
        
        integration_time = []
        for p in range(num_segments):
            integration_time.append(self.NanonisTCP.hex_to_float32(response[idx:idx+4]))
            idx += 4
            
        steps = []
        for p in range(num_segments):
            steps.append(self.NanonisTCP.hex_to_int32(response[idx:idx+4]))
            idx += 4
            
        lockin_run = []
        for p in range(num_segments):
            lockin_run.append(self.NanonisTCP.hex_to_uint32(response[idx:idx+4]))
            idx += 4
        
        retDict = {"bias_start"             : bias_start,
                   "bias_end"               : bias_end,
                   "initial_settling_time"  : initial_settling_time,
                   "settling_time"          : settling_time,
                   "integration_time"       : integration_time,
                   "steps"                  : steps,
                   "lockin_run"             : lockin_run}
        
        return retDict


# -*- coding: utf-8 -*-
"""
Created on Sun May 29 09:02:02 2022

@author: Julian Ceddia
"""

class Current:
    """
    Nanonis Current Module
    """
    def __init__(self,NanonisTCP):
        self.NanonisTCP = NanonisTCP
    
    def Get(self):
        """
        Returns the tunnelling current value

        Returns
        -------
        current : Current value (A)

        """
        ## Make Header
        hex_rep = self.NanonisTCP.make_header('Current.Get', body_size=0)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response(4)
        
        current = self.NanonisTCP.hex_to_float32(response[0:4])
        
        return current
    
    def Get100(self):
        """
        Returns the current value of the "Current 100" module

        Returns
        -------
        current100 : Current 100 value (A)

        """
        ## Make Header
        hex_rep = self.NanonisTCP.make_header('Current.100Get', body_size=0)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response(4)
        
        current100 = self.NanonisTCP.hex_to_float32(response[0:4])
        
        return current100
    
    def BEEMGet(self):
        """
        Returns the BEEM current value of the corresponding module in a BEEM
        system

        Returns
        -------
        currentBEEM : Current BEEM value (A)

        """
        ## Make Header
        hex_rep = self.NanonisTCP.make_header('Current.BEEMGet', body_size=0)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response(4)
        
        currentBEEM = self.NanonisTCP.hex_to_float32(response[0:4])
        
        return currentBEEM
    
    def GainSet(self,gain_index):
        """
        Sets the gain of the current amplifier

        Parameters
        ----------
        gain_index : The index out of the list of gains which can be retrieved
                     by the function Current.GainsGet

        """
        ## Make Header
        hex_rep = self.NanonisTCP.make_header('Current.GainSet', body_size=2)
        
        ## Arguments
        hex_rep += self.NanonisTCP.to_hex(gain_index,2)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
    
    def GainsGet(self):
        """
        Returns the selectable gains of the current amplifier and the index of 
        the selected one

        Returns
        -------
        gains      : array of selectable gains
        gain_index : index of the selected gain in gains array

        """
        ## Make Header
        hex_rep = self.NanonisTCP.make_header('Current.GainsGet', body_size=0)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response()
        
        # gains_size      = self.NanonisTCP.hex_to_int32(response[0:4])         # Not needed since
        number_of_gains = self.NanonisTCP.hex_to_int32(response[4:8])           # We know the number of gains
        
        idx   = 8
        gains = []
        for g in range(number_of_gains):
            size = self.NanonisTCP.hex_to_int32(response[idx:idx+4])            # And the size of each next gain
            idx += 4
            gain = response[idx:idx+size].decode()
            idx += size
            gains.append(gain)
        
        gain_index = self.NanonisTCP.hex_to_uint16(response[idx:idx+2])
        
        return [gains,gain_index]
    
    def CalibrSet(self,calibration,offset):
        """
        Sets the calibration and offset of the selected gain in the current 
        module

        Parameters
        ----------
        calibration : calibration factor (A/V)
        offset      : offset (A)

        """
        ## Make Header
        hex_rep = self.NanonisTCP.make_header('Current.CalibrSet', body_size=16)
        
        ## Arguments
        hex_rep += self.NanonisTCP.float64_to_hex(calibration)
        hex_rep += self.NanonisTCP.float64_to_hex(offset)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
    
    def CalibrGet(self):
        """
        Gets the calibration and offset of the selected gain in the current 
        module

        Returns
        -------
        callibtation : calibration (A/V)
        offset       : offset (A)

        """
        ## Make Header
        hex_rep = self.NanonisTCP.make_header('Current.CalibrGet', body_size=0)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response(16)
        
        calibration = self.NanonisTCP.hex_to_float64(response[0:8])
        offset      = self.NanonisTCP.hex_to_float64(response[8:16])
        
        return [calibration,offset]


# -*- coding: utf-8 -*-
"""
Created on Sat Apr 16 15:39:58 2022

@author: jced0001
"""

class Bias:
    """
    Nanonis Bias Module
    """
    def __init__(self, nanonisTCP):
        self.nanonisTCP = nanonisTCP
    
    def Set(self, bias):
        """
        Set the tip voltage bias

        Parameters
        bias (float32): bias (V)
        """
        ## Make Header
        hex_rep = self.nanonisTCP.make_header('Bias.Set', body_size=4)
        
        ## Arguments
        hex_rep += self.nanonisTCP.float32_to_hex(bias)                         # bias (float 32)
        
        self.nanonisTCP.send_command(hex_rep)
        
        self.nanonisTCP.receive_response(0)
    
    def Get(self):
        """
        Returns the tip bias

        Returns
        bias (float32): bias (V)

        """
        ## Make Header
        hex_rep = self.nanonisTCP.make_header('Bias.Get', body_size=0)
        
        self.nanonisTCP.send_command(hex_rep)
        
        response = self.nanonisTCP.receive_response(4)
        
        bias = self.nanonisTCP.hex_to_float32(response[0:4])
        return bias
    
    def RangeSet(self,bias_range_index):
        """
        Sets the range of the Bias voltage, if different ranges are available

        Parameters
        bias_range_index : is the index out of the list of ranges which can be 
                           retrieved by the function Bias.RangeGet

        """
        ## Make Header
        hex_rep = self.nanonisTCP.make_header('Bias.RangeSet', body_size=2)
        
        ## Arguments
        hex_rep += self.nanonisTCP.to_hex(bias_range_index,2)                   # uint16
        
        self.nanonisTCP.send_command(hex_rep)
        
        # Receive response (check errors)
        self.nanonisTCP.receive_response(0)
        
    def RangeGet(self):
        """
        Returns the selectable ranges of bias voltage and the index of the 
        selected one

        Returns
        bias_ranges      : returns an array of selectable bias ranges (if your
                           system supports it switching. see bias module doco)
        bias_range_index : is the index out of the list of bias ranges

        """
        ## Make Header
        hex_rep = self.nanonisTCP.make_header('Bias.RangeGet', body_size=0)
        
        self.nanonisTCP.send_command(hex_rep)
        
        # Receive response
        response = self.nanonisTCP.receive_response()                           # Not checking for errors because return arguments are variable size
        
        # bias_ranges_size = self.nanonisTCP.hex_to_int32(response[0:4])        # Not needed.
        
        number_of_ranges = self.nanonisTCP.hex_to_int32(response[4:8])
        bias_ranges      = []
        idx = 8
        for i in range(number_of_ranges):
            size = self.nanonisTCP.hex_to_uint32(response[idx:idx+4])
            
            idx += 4
            bias_ranges.append(response[idx:idx+size].decode())
            idx += size
            
        bias_range_index = self.nanonisTCP.hex_to_uint16(response[idx:idx+2])
        
        return [bias_ranges, bias_range_index]
    
    def CalibrSet(self,calibration,offset):
        """
        Set the calibration and offset of bias voltage

        Parameters
        ----------
        calibration : calibration
        offset      : offset

        """
        ## Make Header
        hex_rep = self.nanonisTCP.make_header('Bias.CalibrSet', body_size=8)
        
        ## Arguments
        hex_rep += self.nanonisTCP.float32_to_hex(calibration)
        hex_rep += self.nanonisTCP.float32_to_hex(offset)
        
        self.nanonisTCP.send_command(hex_rep)
        
        # Receive response (check errors)
        self.nanonisTCP.receive_response(0)
    
    def CalibrGet(self):
        """
        Gets the calibration and offset of bias voltage

        Returns
        -------
        calibration : calibration
        offset      : offset

        """
        ## Make Header
        hex_rep = self.nanonisTCP.make_header('Bias.CalibrGet', body_size=0)
        
        self.nanonisTCP.send_command(hex_rep)
        
        # Receive response
        response = self.nanonisTCP.receive_response(8)
        
        calibration = self.nanonisTCP.hex_to_float32(response[0:4])
        offset      = self.nanonisTCP.hex_to_float32(response[4:8])
        
        return [calibration,offset]
    
    def Pulse(self,bias_pulse_width,bias_value,z_hold=0,rel_abs=0,wait_until_done=False):
        """
        Generates one bias pulse

        Parameters
        ----------
        bias_pulse_width : is the pulse duration in seconds
        bias_value       : is the bias value applied during the pulse
        z_hold           : sets whether the controller is set to hold
                           (deactivated) during the pulse.
                           0: no change (leave setting as is in nanonis)
                           1: hold
                           2: don't hold
        rel_abs          : sets whether the bias value argument is an absolute
                           value or relative to the current bias
                           0: no change (leave setting as is in nanonis)
                           1: hold
                           2: absolute
        wait_until_done  : 0: don't wait
                           1: wait until function is done

        """
        ## Make Header
        hex_rep = self.nanonisTCP.make_header('Bias.Pulse', body_size=16)
        
        ## Arguments
        hex_rep += self.nanonisTCP.to_hex(wait_until_done,4)
        hex_rep += self.nanonisTCP.float32_to_hex(bias_pulse_width)
        hex_rep += self.nanonisTCP.float32_to_hex(bias_value)
        hex_rep += self.nanonisTCP.to_hex(z_hold,2)
        hex_rep += self.nanonisTCP.to_hex(rel_abs,2)
        
        self.nanonisTCP.send_command(hex_rep)
        
        # Receive response (check errors)
        self.nanonisTCP.receive_response(0)


# -*- coding: utf-8 -*-
"""
Created on Sun May 29 09:40:40 2022

@author: Julian Ceddia
"""
"""
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!WARNING: RUNNING run_test() WILL CHANGE SETTINGS IN NANONIS. RUN IT ON A SIM!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
"""

from nanonisTCP import nanonisTCP
from nanonisTCP.Current import Current

"""
Set up the TCP connection and interface
"""
def run_test(TCP_IP='127.0.0.1', TCP_PORT=6501):
    # Listening Port: see Nanonis File>Settings>TCP Programming Interface
    NTCP = nanonisTCP(TCP_IP, TCP_PORT)                                         # Nanonis TCP interface
    try:
        currentModule = Current(NTCP)                                           # Nanonis TCP Current Module
        
        """
        Current Get
        """
        current     = currentModule.Get()
        # current100  = currentModule.Get100()
        # currentBEEM = currentModule.BEEMGet()
        print("Current Get")
        print("Current:      " + str(current) + " A")
        # print("100 Current:  " + str(current100))
        # print("BEEM Current: " + str(currentBEEM))
        print("----------------------------------------------------------------------")
        
        """
        Current Gains Get/Set
        """
        currentModule.GainSet(13)
        gains,gain_index = currentModule.GainsGet()
        print("Current Gains")
        print("Gain: " + gains[gain_index])
        print("----------------------------------------------------------------------")
        
        """
        Calibration Set/Get
        """
        currentModule.CalibrSet(calibration=1.1e-9, offset=-1.69e-12)
        calibration,offset = currentModule.CalibrGet()
        print("Calibration")
        print("Calibration Factor: " + str(calibration) + " (A/V)")
        print("Offset:             " + str(offset) + " A")
        print("----------------------------------------------------------------------")
    except:
        print('error')
    NTCP.close_connection()
    return print('end of test')


# -*- coding: utf-8 -*-
"""
Created on Tue May  3 19:04:05 2022

@author: Julian Ceddia
"""

class AutoApproach:
    """
    Nanonis Auto Approach Module
    """
    def __init__(self, NanonisTCP):
        self.NanonisTCP = NanonisTCP
    
    def Open(self):
        """
        Opens the Auto Approach module

        """
        ## Make Header
        hex_rep = self.NanonisTCP.make_header('AutoApproach.Open', body_size=0)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
    
    def OnOffSet(self,on_off):
        """
        Starts or stops the Z auto-approach procedure

        Parameters
        ----------
        on_off : starts or stops Z auto approach
                 False: Stops it
                 True:  Starts it

        """
        ## Make Header
        hex_rep = self.NanonisTCP.make_header('AutoApproach.OnOffSet', body_size=2)
        
        ## Arguments
        hex_rep += self.NanonisTCP.to_hex(on_off,2)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
    
    def OnOffGet(self):
        """
        Returns the on-off status of the Z auto-approach procedure

        Returns
        -------
        on_off : starts or stops Z auto approach
                 False: Off
                 True:  Running

        """
        ## Make Header
        hex_rep = self.NanonisTCP.make_header('AutoApproach.OnOffGet', body_size=0)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response(2)
        
        on_off = self.NanonisTCP.hex_to_uint16(response[0:2])
        
        return on_off


# -*- coding: utf-8 -*-
"""
Created on Mon May 30 12:41:36 2022

@author: Julian Ceddia
"""
"""
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!WARNING: RUNNING run_test() WILL CHANGE SETTINGS IN NANONIS. RUN IT ON A SIM!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
"""

from nanonisTCP import nanonisTCP
from nanonisTCP.Signals import Signals

def run_test(TCP_IP='127.0.0.1', TCP_PORT=6501):
    # Listening Port: see Nanonis File>Settings>TCP Programming Interface
    NTCP = nanonisTCP(TCP_IP, TCP_PORT)                                         # Nanonis TCP interface
    try:
        signalsModule = Signals(NTCP)                                           # Nanonis TCP Signals Module
        
        """
        Signals NamesGet
        """
        signal_names = signalsModule.NamesGet()
        print("Signals NamesGet")
        print("Signal names:\n" + "\n".join(signal_names))
        print("----------------------------------------------------------------------")
        
        """
        Signals InSlots Set/Get
        """
        signalsModule.InSlotSet(slot=5, RTSignalIndex=10)                       # Puts "Input 11 (V)" into slot index 5 of 24
        signal_names,signal_indexes = signalsModule.InSlotsGet()
        print("Signals InSlots")
        for i,signal_name in enumerate(signal_names):
            print("signal " + str(i) + ": " + str(signal_indexes[i]) + "| " + signal_name)
        print("----------------------------------------------------------------------")
        
        """
        Signals CalibrGet
        """
        calibration,offset = signalsModule.CalibrGet(24)
        print("Signals CalibrGet")
        print("Calibration: " + str(calibration))
        print("Offset:      " + str(offset))
        print("----------------------------------------------------------------------")
        
        """
        Signals RangeGet
        """
        max_limit,min_limit = signalsModule.RangeGet(24)
        print("Signals RageGet")
        print("Max limit: " + str(max_limit))
        print("Min limit: " + str(min_limit))
        print("----------------------------------------------------------------------")
        
        """
        Signals GetVal
        """
        signal_val = signalsModule.ValGet(2)
        print("Signals GetVal")
        print("Signal value: " + str(signal_val))
        print("----------------------------------------------------------------------")
    except Exception as e:
        print(str(e))
    NTCP.close_connection()
    return print('end of test')


# -*- coding: utf-8 -*-
"""
Created on Mon Apr 25 15:26:23 2022

@author: Julian Ceddia
"""
"""
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!WARNING: RUNNING run_test() WILL CHANGE SETTINGS IN NANONIS. RUN IT ON A SIM!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
"""


from nanonisTCP import nanonisTCP
from nanonisTCP.FolMe import FolMe

"""
Set up the TCP connection and interface
"""
def run_test(TCP_IP='127.0.0.1', TCP_PORT=6501):
    # Listening Port: see Nanonis File>Settings>TCP Programming Interface
    NTCP = nanonisTCP(TCP_IP, TCP_PORT)                                             # Nanonis TCP interface
    try:
        followme = FolMe(NTCP)                                                          # Nanonis Follow Me Module

        """
        XYPos Set/Get
        """
        followme.XYPosSet(5e-9, -5e-9, Wait_end_of_move=True)                           # Set xy pos to 5 nm, -5 nm
        x,y = followme.XYPosGet(Wait_for_newest_data=True)                              # Get the current XY tip position
        print("Tip Position: " + str(x) + " m, " + str(y) + " m")    

        """
        Speed Set/Get
        """
        followme.SpeedSet(speed=100e-9, custom_speed=True)                              # Set the custom move speed to 100 nm/s. Set move mode to custom speed
        speed, custom_speed = followme.SpeedGet()                                       # Get the custom move speed. get the mode
        print("Speed: " + str(speed) + " m/s")
        print("Mode: " + ["Scan Speed","Custom Speed"][custom_speed])

        """
        Oversampl Set/Get
        """
        followme.OversamplSet(3)                                                        # Set the oversampling for data acquisition in follow me mode
        oversample, sample_rate = followme.OversamplGet()                               # Get the oversampling and sample rate for data acquisition in follow me mode
        print("Oversampling: " + str(oversample) + " @ " + str(sample_rate) + "S/s")

        """
        Stop
        """
        followme.Stop()                                                                 # Stop the tip from moving. no effect if tip position was set with Wait_end_of_move = True

        """
        Point and Shoot Set/Get
        """
        followme.PSOnOffSet(False)                                                      # Disable point and shoot in follow me mode
        ps_status = followme.PSOnOffGet()                                               # Get point and shoot status
        print("Point & Shoot: " + ["disabled","enabled"][ps_status])
    except:
        print('error')

    NTCP.close_connection()
    
    return print('end of test')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Mar 29 16:56:08 2022

@author: Julian Ceddia
"""
"""
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!WARNING: RUNNING run_test() WILL CHANGE SETTINGS IN NANONIS. RUN IT ON A SIM!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
"""


from nanonisTCP import nanonisTCP
from nanonisTCP.Bias import Bias

"""
Set up the TCP connection and interface
"""
def run_test(TCP_IP='127.0.0.1', TCP_PORT=6501):
    # Listening Port: see Nanonis File>Settings>TCP Programming Interface
    NTCP = nanonisTCP(TCP_IP, TCP_PORT)                                             # Nanonis TCP interface
    try:

        """
        Bias Set/Get
        """
        bias = Bias(NTCP)                                                               # Nanonis Bias Module

        bias.Set(5)                                                                     # Set bias to 5 V
        v = bias.Get()                                                                  # Get the current bias
        print("Bias Get")
        print("Bias: " + str(v) + " V")
        print("----------------------------------------------------------------------")

        """
        Range Set/Get
        """
        bias.RangeSet(0)
        bias_ranges, bias_range_index = bias.RangeGet()
        print("Bias Range")
        print("Ranges:      " + str(bias_ranges))
        print("Range index: " + str(bias_range_index))
        print("----------------------------------------------------------------------")

        """
        Calibration Set/Get
        """
        bias.CalibrSet(calibration=2, offset=-0.1)
        calibration,offset = bias.CalibrGet()
        print("Calibration")
        print("Calibration: " + str(calibration))
        print("Offset:      " + str(offset))
        print("----------------------------------------------------------------------")

        """
        Pulse
        """
        bias.Pulse(0.25, -3,z_hold=1,rel_abs=1,wait_until_done=True)
    except:
        print('error')
    NTCP.close_connection()
    return print('end of test')



# -*- coding: utf-8 -*-
"""
Created on Thu May  5 11:14:19 2022

@author: Benjamin Lowe
"""
"""
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!WARNING: RUNNING run_test() WILL CHANGE SETTINGS IN NANONIS. RUN IT ON A SIM!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
"""

from nanonisTCP import nanonisTCP
from nanonisTCP.TipShaper import TipShaper

"""
Set up the TCP connection and interface
"""
def run_test(TCP_IP='127.0.0.1', TCP_PORT=6501, make_plot=False):
    
    # Listening Port: see Nanonis File>Settings>TCP Programming Interface
    NTCP = nanonisTCP(TCP_IP, TCP_PORT)                                         # Nanonis TCP interface
    
    tipShaper = TipShaper(NTCP)                                                 # Nanonis Tip Shaper Module
    
    try:
              
        """
        Properties Set/Get
        """
        tipShaper.PropsSet(Switch_off_delay=1,Change_bias=1,Bias=0.02,Tip_lift=-2e-9,Lift_time_1=0.2,Bias_lift=0.05,
                Bias_settling_time=0.1,Lift_height=5e-9,Lift_time_2=0.2,End_wait_time=0.1,
                Restore_feedback=1)
        
        Switch_off_delay,Change_bias,Bias,Tip_lift,Lift_time_1,Bias_lift,Bias_settling_time,Lift_height,Lift_time_2,End_wait_time,Restore_feedback = tipShaper.PropsGet()
        print("Tip Shaper Properties")
        print("Switch off delay:  " + "%f" % Switch_off_delay)
        print("Change Bias status:" + ["no change","True","False"][Change_bias])
        print("Bias: %f" % Bias)
        print("Tip Lift: %f m" % Tip_lift)
        print("Lift_time_1: %f s" % Lift_time_1)
        print("Bias_lift: %f V" % Bias_lift)
        print("Bias settling time: %f s" % Bias_settling_time)
        print("Lift_height: %f m" % Lift_height)
        print("Lift_time_2: %f s" % Lift_time_2)
        print("End_wait_time: %f" % End_wait_time)
        print("Restore Feedback?" + ["no change","yes","no"][Restore_feedback])
        print("----------------------------------------------------------------------")
        
        """
        Start
        """
        tipShaper.Start(wait_until_finished=1,timeout=10)
        
    except Exception as e:
        print(e)
    
    NTCP.close_connection()


# -*- coding: utf-8 -*-
"""
Created on Tue Oct 18 19:20:57 2022

@author: jced0001
"""
"""
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!WARNING: RUNNING run_test() WILL CHANGE SETTINGS IN NANONIS. RUN IT ON A SIM!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
"""

from nanonisTCP import nanonisTCP
from nanonisTCP.LockIn import LockIn
import traceback

def run_test(TCP_IP='127.0.0.1', TCP_PORT=6501):
    # Listening Port: see Nanonis File>Settings>TCP Programming Interface
    NTCP = nanonisTCP(TCP_IP, TCP_PORT)                                         # Nanonis TCP interface
    try:
        lockin = LockIn(NTCP)                                                   # Nanonis TCP Lock In Module
        
        """
        On/Off Set/Get
        """
        lockin.ModOnOffSet(modulator_number=1,lockin_onoff=1)
        lockin1 = lockin.ModOnOffGet(modulator_number=1)
        lockin2 = lockin.ModOnOffGet(modulator_number=2)
        print("lockin1: " + ["off","on"][lockin1])
        print("lockin2: " + ["off","on"][lockin2])
        print("----------------------------------------------------------------------")
        
        """
        ModSignalSet/Get
        """
        lockin.ModSignalSet(modulator_number=1,modulator_signal_index=24)       # Make sure the signal index you give is available with Signals.NamesGet
        modulator_signal_index1  = lockin.ModSignalGet(modulator_number=1)
        modulator_signal_index2  = lockin.ModSignalGet(modulator_number=2)
        print("Signal1: " + str(modulator_signal_index1))
        print("Signal2: " + str(modulator_signal_index2))
        print("----------------------------------------------------------------------")
        
        """
        ModPhasRegSet/Get
        """
        try:
            lockin.ModPhasRegSet(modulator_number=1,phase_register_index=2)     # Access denied error in demo mode
        except Exception as e:
            print("Could not set phase register. " + str(e))
        phase_reg1 = lockin.ModPhasRegGet(modulator_number=1)
        phase_reg2 = lockin.ModPhasRegGet(modulator_number=2)
        print("Signal1: " + str(phase_reg1))
        print("Signal2: " + str(phase_reg2))
        print("----------------------------------------------------------------------")
        
        """
        ModHarmonicSet/Get
        """
        try:
            lockin.ModHarmonicSet(modulator_number=1,harmonic=2)                # Access denied error in demo mode
        except Exception as e:
            print("Could not set phase register. " + str(e))
        harmonic1 = lockin.ModHarmonicGet(modulator_number=1)
        harmonic2 = lockin.ModHarmonicGet(modulator_number=2)
        print("Harmonic1: " + str(harmonic1))
        print("Harmonic2: " + str(harmonic2))
        print("----------------------------------------------------------------------")
        
        """
        ModPhasSet/Get
        """
        try:
            lockin.ModPhasSet(modulator_number=1,phase_deg=99)                  # Access denied error in demo mode
        except Exception as e:
            print("Could not set phase. " + str(e))
        phase1 = lockin.ModPhasGet(modulator_number=1)
        phase2 = lockin.ModPhasGet(modulator_number=2)
        print("Phase1: " + str(phase1))
        print("Phase2: " + str(phase2))
        print("----------------------------------------------------------------------")
        
        """
        ModAmpSet/Get
        """
        try:
            lockin.ModAmpSet(modulator_number=1,amplitude=2)                    # Access denied error in demo mode
        except Exception as e:
            print("Could not set amplitude. " + str(e))
        amplitude1 = lockin.ModAmpGet(modulator_number=1)
        amplitude2 = lockin.ModAmpGet(modulator_number=2)
        print("Amplitude1: " + str(amplitude1))
        print("Amplitude2: " + str(amplitude2))
        print("----------------------------------------------------------------------")
        
        """
        ModPhasFreqSet/Get
        """
        try:
            lockin.ModPhasFreqSet(modulator_number=1,frequency=2e3)             # Access denied error in demo mode
        except Exception as e:
            print("Could not set frequency. " + str(e))
        frequency1 = lockin.ModPhasFreqGet(modulator_number=1)
        frequency2 = lockin.ModPhasFreqGet(modulator_number=2)
        print("Frequency1: " + str(frequency1))
        print("frequency2: " + str(frequency2))
        print("----------------------------------------------------------------------")
        
        """
        DemodSignalSet/Get
        """
        lockin.DemodSignalSet(demod_number=1,demod_signal_index=1)
        demod_signal_index1 = lockin.DemodSignalGet(demod_number=1)
        demod_signal_index2 = lockin.DemodSignalGet(demod_number=2)
        print("Signal1: " + str(demod_signal_index1))
        print("Signal2: " + str(demod_signal_index2))
        print("----------------------------------------------------------------------")
        
        """
        DemodLPFilterSet/Get
        """
        try:
            lockin.DemodLPFilterSet(demod_number=1,lp_filter_order=1,cutoff=2e3)# Access denied error in demo mode
        except Exception as e:
            print("Could not set lpfilter. " + str(e))
        lpfilter1 = lockin.DemodLPFilterGet(demod_number=1)
        lpfilter2 = lockin.DemodLPFilterGet(demod_number=2)
        print("LP Filter1: " + str(lpfilter1))
        print("LP Filter2: " + str(lpfilter2))
        print("----------------------------------------------------------------------")
        
        """
        DemodPhasRegSet/Get
        """
        try:
            lockin.DemodPhasRegSet(demod_number=1,phase_register_index=1)       # Access denied error in demo mode
        except Exception as e:
            print("Could not set phase register. " + str(e))
        phase_reg1 = lockin.DemodPhasRegGet(demod_number=1)
        phase_reg2 = lockin.DemodPhasRegGet(demod_number=2)
        print("Phase Register1: " + str(phase_reg1))
        print("Phase Register2: " + str(phase_reg2))
        print("----------------------------------------------------------------------")
        
        """
        DemodPhasSet/Get
        """
        try:
            lockin.DemodPhasSet(demod_number=1,phase_deg=98)                    # Access denied error in demo mode
        except Exception as e:
            print("Could not set phase. " + str(e))
        phase1 = lockin.DemodPhasGet(demod_number=1)
        phase2 = lockin.DemodPhasGet(demod_number=2)
        print("Phase1: " + str(phase1))
        print("Phase2: " + str(phase2))
        print("----------------------------------------------------------------------")
        
        """
        DemodSyncFilterSet/Get
        """
        try:
            lockin.DemodSyncFilterSet(demod_number=1,sync_filter=0)             # Access denied error in demo mode
        except Exception as e:
            print("Could not set sync filter. " + str(e))
        syncFilter1 = lockin.DemodSyncFilterGet(demod_number=1)
        syncFilter2 = lockin.DemodSyncFilterGet(demod_number=2)
        print("syncFilter1: " + str(syncFilter1))
        print("syncFilter2: " + str(syncFilter2))
        print("----------------------------------------------------------------------")
        
        """
        DemodRTSignalsSet/Get
        """
        try:
            lockin.DemodRTSignalsSet(demod_number=1,rt_signals=1)               # Access denied error in demo mode
        except Exception as e:
            print("Could not set RT Signals. " + str(e))
        rtSignals1 = lockin.DemodRTSignalsGet(demod_number=1)
        rtSignals2 = lockin.DemodRTSignalsGet(demod_number=2)
        print("rtSignals1: " + str(rtSignals1))
        print("rtSignals2: " + str(rtSignals2))
        print("----------------------------------------------------------------------")
    except:
        print(traceback.format_exc())
        
    NTCP.close_connection()
    return print('end of test')
        


# -*- coding: utf-8 -*-
"""
Created on Mon May 30 12:33:59 2022

@author: Julian Ceddia
"""

class Signals:
    """
    Nanonis Signals Module
    """
    def __init__(self,NanonisTCP):
        self.NanonisTCP = NanonisTCP
    
    def NamesGet(self):
        """
        Returns the signals names list of the 128 signals available in the 
        software

        Returns
        -------
        signal_names : String array of signal names

        """
        ## Make Header
        hex_rep = self.NanonisTCP.make_header('Signals.NamesGet', body_size=0)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response()
        
        # signals_names_size = self.NanonisTCP.hex_to_int32(response[0:4])
        signals_names_num  = self.NanonisTCP.hex_to_int32(response[4:8])
        
        idx = 8
        signal_names = []
        for n in range(signals_names_num):
            size = self.NanonisTCP.hex_to_int32(response[idx:idx+4])
            idx += 4
            signal_name = response[idx:idx+size].decode()
            idx += size
            signal_names.append(signal_name)
        
        return signal_names
    
    def InSlotSet(self,slot,RTSignalIndex):
        """
        Assigns one of the 128 available signals to one of the 24 slots of the 
        Signals Manager.

        Parameters
        ----------
        slot          : The index of the slot in the Signals Manager where one 
                        of the 128 RT signals is assigned, so that index could 
                        be any value from 0 to 23
        RTSignalIndex : The index of the RT signal to assign to the selected 
                        slot, so that index could be any value from 0 to 127

        """
        ## Make Header
        hex_rep = self.NanonisTCP.make_header('Signals.InSlotSet', body_size=8)
        
        ## Arguments
        hex_rep += self.NanonisTCP.to_hex(slot,4)
        hex_rep += self.NanonisTCP.to_hex(RTSignalIndex,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        self.NanonisTCP.receive_response(0)
    
    def InSlotsGet(self):
        """
        Returns a list of the signals names and indexes of the 24 signals 
        assigned to the slots of the Signals Manager.
        
        The 24 signals are selected in the Signals Manager out of the 128 
        signals available in the software, and they are used in the list of 
        available signals to display in graphs and other modules.
        
        The index of every signal corresponds to the index in the list of 128 
        signals (0-127). The latter can be returned by Signals.NamesGet.

        Returns
        -------
        signal_names   : Str array of selected signal names
        signal_indexes : Int array of signal indexes

        """
        ## Make Header
        hex_rep = self.NanonisTCP.make_header('Signals.InSlotsGet', body_size=0)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response()
        
        # signals_names_size = self.NanonisTCP.hex_to_int32(response[0:4])
        signals_names_num  = self.NanonisTCP.hex_to_int32(response[4:8])
        
        idx = 8
        signal_names = []
        for n in range(signals_names_num):
            size = self.NanonisTCP.hex_to_int32(response[idx:idx+4])
            idx += 4
            signal_name = response[idx:idx+size].decode()
            idx += size
            signal_names.append(signal_name)
        
        signal_indexes = []
        signal_indexes_size = self.NanonisTCP.hex_to_int32(response[idx:idx+4])
        for n in range(signal_indexes_size):
            idx += 4
            signal_index = self.NanonisTCP.hex_to_int32(response[idx:idx+4])
            signal_indexes.append(signal_index)
            
        return [signal_names,signal_indexes]
    
    def CalibrGet(self,signal_index):
        """
        Returns the calibration and offset of the selected signal.

        Parameters
        ----------
        signal_index : Index of the signal to retrieve calibration for (0-127)

        Returns
        -------
        calibration : signal calibration (V^-1)
        offset      : signal offset in physical units

        """
        ## Make Header
        hex_rep = self.NanonisTCP.make_header('Signals.CalibrGet', body_size=4)
        
        ## Arguments
        hex_rep += self.NanonisTCP.to_hex(signal_index,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response(8)
        
        calibration = self.NanonisTCP.hex_to_float32(response[0:4])
        offset      = self.NanonisTCP.hex_to_float32(response[4:8])
        
        return [calibration,offset]
    
    def RangeGet(self,signal_index):
        """
        Returns the range limits of the selected signal

        Parameters
        ----------
        signal_index : Index of the signal to retrieve range for

        Returns
        -------
        max_limit: range max
        min_limit: range min

        """
        ## Make Header
        hex_rep = self.NanonisTCP.make_header('Signals.RangeGet', body_size=4)
        
        ## Arguments
        hex_rep += self.NanonisTCP.to_hex(signal_index,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response(8)
        
        max_limit = self.NanonisTCP.hex_to_float32(response[0:4])
        min_limit = self.NanonisTCP.hex_to_float32(response[4:8])
        
        return [max_limit,min_limit]
    
    def ValGet(self,signal_index,wait_for_newest_data=True):
        """
        Returns the current value of the selected signal (oversampled during 
        the Acquisition Period time, Tap).
        
        Signal measurement principle:
        The signal is continuously oversampled with the Acquisition Period
        time, Tap, specified in the TCP receiver module. Every Tap second, the 
        oversampled data is "published". This VI function waits for the next 
        oversampled data to be published and returns its value. Calling this 
        function does not trigger a signal measurement; it waits for data to be
        published! Thus, this function returns a value 0 to Tap second after
        being called.
        
        An important consequence is that if you change a signal and immediately
        call this function to read a measurement you might get "old" data
        (i.e. signal data measured before you changed the signal).
        The solution to get only new data is to set Wait for newest data to
        True. In this case, the first published data is discarded and only the
        second one is returned.

        Parameters
        ----------
        signal_index         : Index of the signal to retrieve a value for
        wait_for_newest_data : Selects whether the function returns the next
                               available signal value or if it waits for a full
                               period of new data. If False, this function
                               returns a value 0 to Tap seconds after being
                               called. If True, the function discard the first
                               oversampled signal value received but returns
                               the second value received. Thus, the function
                               returns a value Tap to 2*Tap seconds after being
                               called. It could be 0=False or 1=True

        Returns
        -------
        signal_value : The value of the selected signal in physical units

        """
        ## Make Header
        hex_rep = self.NanonisTCP.make_header('Signals.ValGet', body_size=8)
        
        ## Arguments
        hex_rep += self.NanonisTCP.to_hex(signal_index,4)
        hex_rep += self.NanonisTCP.to_hex(wait_for_newest_data,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        response = self.NanonisTCP.receive_response(8)
        
        signal_value = self.NanonisTCP.hex_to_float32(response[0:4])
        
        return signal_value


# -*- coding: utf-8 -*-
"""
Created on Fri Oct 21 08:21:56 2022

@author: jced0001
"""

import numpy as np

class Marks:
    """
    Nanonis Marks In Scan Module
    """
    def __init__(self,NanonisTCP):
        self.NanonisTCP = NanonisTCP
    
    def PointDraw(self,p,text,c=0):
        """
        Draws text at the specified point of the scan frame.
        This function can be very useful to mark an important location in the
        scan image (i.e. the position where the Tip Shaper executed).

        Parameters
        ----------
        p    : (x,y) coordinate (m) of the centre of the text
        text : Text characters to draw
        c    : RGB text colour (uint32)

        """
        text_size =  int(len(self.NanonisTCP.string_to_hex(text))/2)
        
        body_size = 16 + text_size
        
        ## Make Header
        hex_rep = self.NanonisTCP.make_header('Marks.PointDraw', body_size=body_size)
        
        hex_rep += self.NanonisTCP.float32_to_hex(p[0])
        hex_rep += self.NanonisTCP.float32_to_hex(p[1])
        hex_rep += self.NanonisTCP.to_hex(text_size,4)
        hex_rep += self.NanonisTCP.string_to_hex(text)
        hex_rep += self.NanonisTCP.to_hex(c,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        # Receive Response (check errors)
        self.NanonisTCP.receive_response(0)
        
    def LineDraw(self,start,end,c=0):
        """
        Draw a line in the scan frame

        Parameters
        ----------
        start : Start point (x,y) (m)
        end   : End point (x,y) (m)
        c     : Line colour (uint32)

        """
        ## Make Header
        hex_rep = self.NanonisTCP.make_header('Marks.LineDraw', body_size=20)
        
        hex_rep += self.NanonisTCP.float32_to_hex(start[0])
        hex_rep += self.NanonisTCP.float32_to_hex(start[1])
        hex_rep += self.NanonisTCP.float32_to_hex(end[0])
        hex_rep += self.NanonisTCP.float32_to_hex(end[1])
        hex_rep += self.NanonisTCP.to_hex(c,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        # Receive Response (check errors)
        self.NanonisTCP.receive_response(0)
        
    def PointsErase(self,point_index=-1):
        """
        Erase the point specified by the index parameter from the scan frame.
        -1 erases all points

        Parameters
        ----------
        point_index : sets the index of the point to erase. The index is 
        comprised between 0 and the total number of drawn points minus one. To 
        see which point has which index, use the Marks.PointsGet function. 
        Value -1 erases all points.

        """
        ## Make Header
        hex_rep = self.NanonisTCP.make_header('Marks.PointsErase', body_size=4)
        
        hex_rep += self.NanonisTCP.to_hex(point_index,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        # Receive Response (check errors)
        self.NanonisTCP.receive_response(0)
        
    def LinesErase(self,point_index=-1):
        """
        Erase the line specified by the index parameter from the scan frame.
        -1 erases all lines

        Parameters
        ----------
        point_index : sets the index of the line to erase. The index is 
        comprised between 0 and the total number of drawn lines minus one. 
        To see which line has which index, use the Marks.LinesGet function.
        Value -1 erases all lines.

        """
        ## Make Header
        hex_rep = self.NanonisTCP.make_header('Marks.LinesErase', body_size=4)
        
        hex_rep += self.NanonisTCP.to_hex(point_index,4)
        
        self.NanonisTCP.send_command(hex_rep)
        
        # Receive Response (check errors)
        self.NanonisTCP.receive_response(0)
        


# -*- coding: utf-8 -*-
"""
Created on Wed Aug 24 23:12:28 2022

@author: Julian
"""
"""
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!WARNING: RUNNING run_test() WILL CHANGE SETTINGS IN NANONIS. RUN IT ON A SIM!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
"""

from nanonisTCP import nanonisTCP
from nanonisTCP.BiasSpectr import BiasSpectr
import traceback
import numpy as np

def run_test(TCP_IP='127.0.0.1', TCP_PORT=6501, plot_data=0):
    # Listening Port: see Nanonis File>Settings>TCP Programming Interface
    NTCP = nanonisTCP(TCP_IP, TCP_PORT)                                         # Nanonis TCP interface
    try:
        bspec = BiasSpectr(NTCP)                                                # Nanonis TCP Bias Spectroscopy Module
        
        """
        Open
        """
        bspec.Open()
        print("Bias Spectroscopy module open")
        print("----------------------------------------------------------------------")
        
        """
        Start
        """
        data = bspec.Start(get_data=1,save_base_name="NTCP-Test-")
        data_dict  = data['data_dict']
        parameters = data['parameters']
        if(plot_data):
            import matplotlib.pyplot as plt
            plt.plot(data_dict['Bias calc (V)'],data_dict['Current (A)'])
        print("data channels: " + str(data_dict.keys()))
        print("parameters:    " + str(parameters))
        print("----------------------------------------------------------------------")
        
        """
        Stop
        """
        # Cannot test the Stop function like below... the reason is nanonis 
        # server side sends a response back over the initiating port and cannot
        # receive another instruction (i.e. Stop) until the data acquisition is
        # complete (even if get_data=0). This means if you want to start an 
        # acquisition and then stop it before it finishes, you must send the 
        # Stop command on a separate port.
        
        # bspec.Start(get_data=0,save_base_name="NTCP-Test-Stop-")              # Can't do this to test. must be started and stopped on separate ports
        
        bspec.Stop()
        print("Acquisition stopped")
        print("----------------------------------------------------------------------")
        
        """
        Status Get
        """
        status = bspec.StatusGet()
        print("Status: " + ["Not running","Running"][status])
        print("----------------------------------------------------------------------")
        
        """
        ChsSet/Get
        """
        channel_indexes = [0,14,16]
        bspec.ChsSet(channel_indexes=channel_indexes,mode="set")
        channel_indexes = bspec.ChsGet()
        print("channels : " + str(channel_indexes))
        print("----------------------------------------------------------------------")
        
        """
        Props Set/Get
        """
        bspec.PropsSet(save_all=1,num_sweeps=20,back_sweep=0,num_points=200,z_offset=10e-12,autosave=0,save_dialog=2)
        propsDict = bspec.PropsGet()
        print("Properties:\n" + str(propsDict))
        print("----------------------------------------------------------------------")
        
        """
        AdvPropsSet/Get
        """
        bspec.AdvPropsSet(reset_bias=0,z_controller_hold=0,record_final_z=0,lockin_run=0)
        advancedProps = bspec.AdvPropsGet()
        print("Advanced Properties:\n" + str(advancedProps))
        print("----------------------------------------------------------------------")
        
        """
        LimitsSet/Get
        """
        bspec.LimitsSet(start_value=-1.1,end_value=1.1)
        limits = bspec.LimitsGet()
        print("Spec limits:\n" + str(limits))
        print("----------------------------------------------------------------------")
        
        """
        TimingSet/Get
        """
        # Note: minimum times are 50us. anything lower than this will either 
        # round up to 50us or down to 0s
        bspec.TimingSet(z_averaging_time=51e-5,z_offset=1.2e-9,initial_settling_time=370e-6,maximum_slew_rate=np.inf,settling_time=12e-6,integration_time=57e-6,end_settling_time=91e-6,z_control_time=61e-6)
        timing = bspec.TimingGet()
        print("Spec Timing:\n" + str(timing))
        print("----------------------------------------------------------------------")
        
        """
        AltZCtrlSet/Get
        """
        bspec.AltZCtrlSet(alternate_setpoint_onoff=1,setpoint=22e-12,settling_time=60e-3)
        altZParams = bspec.AltZCtrlGet()
        print("Spec AltZParams:\n" + str(altZParams))
        print("----------------------------------------------------------------------")
        
        """
        MLSLockinPerSegSet/Get and MLSModeSet/Get
        """
        # This test will fail if sweep mode is not explicitly set to 
        # Multi Segment first.
        bspec.MLSModeSet(sweep_mode="MLS")
        sweep_mode = bspec.MLSModeGet()
        print("Sweep mode: " + sweep_mode)
        bspec.MLSLockinPerSegSet(lockin_per_segment=0)
        lockin_per_segment = bspec.MLSLockinPerSegGet()
        print("Spec lockin per segment: " + ["off","on"][lockin_per_segment])
        print("----------------------------------------------------------------------")
        
        """
        MLSValsSet/Get
        """
        bias_start = [-2,-1]
        bias_end   = [1,2]
        initial_settling_time = [10e-3,12e-3]
        settling_time = [14e-3,16e-3]
        integration_time = [18e-3,20e-3]
        steps = [10,20]
        lockin_run=[0,0]
        bspec.MLSValsSet(bias_start,bias_end,initial_settling_time,settling_time,integration_time,steps,lockin_run)
        mlsVals = bspec.MLSValsGet()
        print("MLS Values:\n" + str(mlsVals))
        print("Setting sweep mode back to Linear...")
        bspec.MLSModeSet(sweep_mode="Linear")
        sweep_mode = bspec.MLSModeGet()
        print("Sweep mode: " + sweep_mode)
        print("----------------------------------------------------------------------")
    except:
        print(traceback.format_exc())
        
    NTCP.close_connection()
    return print('end of test')
        



# -*- coding: utf-8 -*-
"""
Created on Tue May  3 19:34:49 2022

@author: Julian Ceddia
"""
"""
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!WARNING: RUNNING run_test() WILL CHANGE SETTINGS IN NANONIS. RUN IT ON A SIM!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
"""

from nanonisTCP import nanonisTCP
from nanonisTCP.Motor import Motor

"""
Set up the TCP connection and interface
"""
def run_test(TCP_IP='127.0.0.1', TCP_PORT=6502, make_plot=False):
    # Listening Port: see Nanonis File>Settings>TCP Programming Interface
    NTCP = nanonisTCP(TCP_IP, TCP_PORT)                                         # Nanonis TCP interface
    
    motor = Motor(NTCP)                                                         # Nanonis Coarse Motion (Motor)Module
    
    try:
        """
        Start Move
        """
        motor.StartMove(direction="Z+",steps=100,group=2,wait_until_finished=False)
        
        """
        Stop Moter Move
        """
        motor.StopMove()
        
        print("----------------------------------------------------------------------")
        """
        Start Closed Loop
        """
        try:
            motor.StartClosedLoop(abs_rel=True, target_x=0, target_y=0, target_z=10, wait_until_finished=False)
        except Exception as e:
            print("Error in Motor.StartClosedLoop: ")
            print(e)
            print("----------------------------------------------------------------------")
        """
        Motor Pos Get
        """
        try:
            x_pos,y_pos,z_pos = motor.PosGet()
            print("Motor Positions")
            print("x pos: " + str(x_pos))
            print("y pos: " + str(y_pos))
            print("z pos: " + str(z_pos))
            print("----------------------------------------------------------------------")
        except Exception as e:
            print("Error in Motor.PosGet: ")
            print(e)
            print("----------------------------------------------------------------------")
        
        """
        Step Counter Get
        """
        try:
            counter_x,counter_y,counter_z = motor.StepCounterGet()
            print("Motor Step Counters")
            print("x steps: " + str(counter_x))
            print("y steps: " + str(counter_y))
            print("z steps: " + str(counter_z))
            print("----------------------------------------------------------------------")
        except Exception as e:
            print("Error in Motor.StepCounterGet: ")
            print(e)
            print("----------------------------------------------------------------------")
        
        """
        Freq/Amp Set/Get
        """
        motor.FreqAmpSet(frequency=1111, amplitude=199)
        [frequency,amplitude] = motor.FreqAmpGet()
        print("Motor Frequency/Amplidtude")
        print("Frequency: " + str(frequency))
        print("Amplitude: " + str(amplitude))
        print("----------------------------------------------------------------------")
        
        print("test complete")
    except Exception as e:
        print(e)
    
    NTCP.close_connection()


# -*- coding: utf-8 -*-
"""
Created on Fri Jan 20 08:41:34 2023

@author: jced0001
"""
"""
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!WARNING: RUNNING run_test() WILL CHANGE SETTINGS IN NANONIS. RUN IT ON A SIM!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
"""
import time
from nanonisTCP import nanonisTCP
from nanonisTCP.Osci2T import Osci2T
import traceback

def run_test(TCP_IP='127.0.0.1', TCP_PORT=6501, plotData=1):
    # Listening Port: see Nanonis File>Settings>TCP Programming Interface
    NTCP = nanonisTCP(TCP_IP, TCP_PORT)                                         # Nanonis TCP interface
    try:
        osci2T = Osci2T(NTCP)                                                   # Nanonis TCP Oscilloscope 2-Channels Module
        """
        Osci2T Run
        """
        osci2T.Run()
        print("Osci2T Running")
        print("----------------------------------------------------------------------")
        
        """
        Osci2T ChsSet/Get
        """
        osci2T.ChsSet(chA=0,chB=24)
        chA,chB = osci2T.ChsGet()
        print("Channel A Index: ",chA)
        print("Channel B Index: ",chB)
        print("----------------------------------------------------------------------")
        
        """
        Osci2T TimebaseSet/Get
        """
        osci2T.TimebaseSet(timebaseIndex=5)
        timebaseIndex,timebases = osci2T.TimebaseGet()
        print("timebaseIndex: ",timebaseIndex)
        print("timebases: ",timebases)
        print("----------------------------------------------------------------------")
        
        """
        Oversample Set/Get
        """
        osci2T.OversamplSet(oversamplingIndex=4)
        oversamplingIndex = osci2T.OversamplGet()
        print("Oversampling Index",oversamplingIndex)
        print("----------------------------------------------------------------------")
        
        """
        Trig Set/Get
        """
        time.sleep(1)                                                           # Updating trigger too soon sometimes causes Nanonis not to register for some reason?
        osci2T.TrigSet(mode=0, channel=0, slope=1, level=0, hysteresis=0, position=2)
        mode,channel,slope,level,hysteresis,position = osci2T.TrigGet()
        print("Mode",mode)
        print("Channel",channel)
        print("slope",slope)
        print("level",level)
        print("hysteresis",hysteresis)
        print("position",position)
        print("----------------------------------------------------------------------")
        
        """
        DataGet
        """
        time.sleep(2) # Wait for data
        t0,dt,chA,chB = osci2T.DataGet(dataToGet=0)
        if(plotData):
            import matplotlib.pyplot as plt
            import numpy as np
            xx = np.arange(len(chA))*dt + t0
            plt.plot(xx,chA)
            plt.plot(xx,chB)
            
        
    except Exception as e:
        print(traceback.format_exc())
        print(e)
    NTCP.close_connection()
    return print('end of test')